from traitlets import (
    Unicode, Dict, Float, Bool, Union, Any,
)
from ..basewidget import BaseWidget


class Map(BaseWidget):
    """ 
:warning: **Autogenerated class**

**Map.**


Map is mainly used in the visualization of geographic area data, which can be used with [visualMap](#visualMap) component to visualize the data such as population distribution density in different areas.


Series of same [map type](#series-map.map) will show in one map. At this point, the configuration of the first series will be used for the map configuration.



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("MapModel").tag(sync=True)
    
    type = Unicode("map", allow_none=True, ).tag(sync=True)
    
    id = Unicode(None, allow_none=True, help="""Component ID, not specified by default. If specified, it can be used to refer the component in option or API.""").tag(sync=True)
    
    name = Unicode(None, allow_none=True, help="""Series name used for displaying in [tooltip](#tooltip) and filtering with [legend](#legend), or updating data and configuration with `setOption`.""").tag(sync=True)
    
    colorBy = Unicode(None, allow_none=True, help="""> Since `v5.2.0`


The policy to take color from [option.color](#color). Valid values:


* `'series'`: assigns the colors in the palette by series, so that all data in the same series are in the same color;
* `'data'`: assigns colors in the palette according to data items, with each data item using a different color.""").tag(sync=True)
    
    map = Unicode(None, allow_none=True, help="""Map name registered in [registerMap](api.html#echarts.registerMap).


**Use geoJSON**



```
$.get('map/china_geo.json', function (geoJson) {
    echarts.registerMap('china', {geoJSON: geoJson});
    var chart = echarts.init(document.getElementById('main'));
    chart.setOption({
        series: [{
            type: 'map',
            map: 'china',
            ...
        }]
    });
});

```

See also [USA Population Estimates](https://echarts.apache.org/examples/en/editor.html?c=map-usa).


The demo above shows that ECharts can uses [geoJSON](http://geojson.org/) format as map outline. You can use third-party [geoJSON](http://geojson.org/) data (like [maps](https://github.com/echarts-maps)) and register them into ECharts.


**Use SVG**



```
$.get('map/topographic_map.svg', function (svg) {
    echarts.registerMap('topo', {svg: svg});
    var chart = echarts.init(document.getElementById('main'));
    chart.setOption({
        series: [{
            type: 'map',
            map: 'topo',
            ...
        }]
    });
});

```

See also [Beef Cuts](https://echarts.apache.org/examples/en/editor.html?c=geo-beef-cuts).


The demo above shows that SVG format can be used in ECharts. See more info in [SVG Base Map](tutorial.html#SVG%20Base%20Map%20in%20Geo%20Coords%20and%20Map%20Series).""").tag(sync=True)
    
    roam = Union([Bool(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Whether to enable mouse zooming and translating. `false` by default. If either zooming or translating is wanted, it can be set to `'scale'` or `'move'`. Otherwise, set it to be `true` to enable both.""").tag(sync=True)
    
    projection = Dict(default_value=None, allow_none=True, help="""> Since `v5.3.0`


For custom map projection, at least two methods `project`, `unproject` should be provided to calculate the coordinates after projection and before projection respectively.


For example, for the Mercator projection.



```
series: {
    type: 'map',
    projection: {
        project: (point) => [point[0] / 180 * Math.PI, -Math.log(Math.tan((Math.PI / 2 + point[1] / 180 * Math.PI) / 2))],
        unproject: (point) => [point[0] * 180 / Math.PI, 2 * 180 / Math.PI * Math.atan(Math.exp(point[1])) - 90]
    }
}

```

In addition to our own implementation of the projection formula, we can also use exists projection implementations provided by third-party libraries such as [d3-geo](https://github.com/d3/d3-geo).



```
const projection = d3.geoConicEqualArea();
// ...
series: {
    type: 'map',
    projection: {
        project: (point) => projection(point),
        unproject: (point) => projection.invert(point)
    }
}


```

Note: Custom projections are only useful when using `GeoJSON` as a data source.""").tag(sync=True)
    
    center = Any(None, allow_none=True, help="""Center of current view-port. It can be an array containing two `number`s in pixels or `string`s in percentage relative to the container width/height.
`string` is supported from version `5.3.3`.


Example:



```
center: [115.97, '30%']

```""").tag(sync=True)
    
    aspectScale = Float(None, allow_none=True, help="""Used to scale aspect of geo. Will be ignored if `projection` is set.


The final aspect is calculated by: `geoBoundingRect.width / geoBoundingRect.height * aspectScale`.""").tag(sync=True)
    
    boundingCoords = Any(None, allow_none=True, help="""Two dimension array. Define coord of left-top, right-bottom in layout box.



```
// A complete world map
map: 'world',
left: 0, top: 0, right: 0, bottom: 0,
boundingCoords: [
    // [lng, lat] of left-top corner
    [-180, 90],
    // [lng, lat] of right-bottom corner
    [180, -90]
],

```""").tag(sync=True)
    
    zoom = Float(None, allow_none=True, help="""Zoom rate of current view-port.""").tag(sync=True)
    
    scaleLimit = Dict(default_value=None, allow_none=True, help="""Limit of scaling, with `min` and `max`. `1` by default.""").tag(sync=True)
    
    nameMap = Dict(default_value=None, allow_none=True, help="""Name mapping for customized areas. For example:



```
{
    'China' : '中国'
}

```""").tag(sync=True)
    
    nameProperty = Unicode(None, allow_none=True, help="""> Since `v4.8.0`


customized property key for GeoJSON feature. By default, 'name' is used as primary key to identify GeoJSON feature.
For example:



```
{
    nameProperty: 'NAME', // key to connect following data point to GeoJSON region {"type":"Feature","id":"01","properties":{"NAME":"Alabama"}, "geometry": { ... }}
    data:[
        {name: 'Alabama', value: 4822023},
        {name: 'Alaska', value: 731449},
    ]
}

```""").tag(sync=True)
    
    selectedMode = Union([Bool(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Selected mode decides whether multiple selecting is supported. By default, `false` is used for disabling selection. Its value can also be `'single'` for selecting single area, or `'multiple'` for selecting multiple areas.""").tag(sync=True)
    
    label = Dict(default_value=None, allow_none=True, help="""Text label of , to explain some data information about graphic item like value, name and so on. `label` is placed under `itemStyle` in ECharts 2.x. In ECharts 3, to make the configuration structure flatter, `label`is taken to be at the same level with `itemStyle`, and has `emphasis` as `itemStyle` does.""").tag(sync=True)
    
    itemStyle = Dict(default_value=None, allow_none=True, help="""Graphic style of Map Area Border, `emphasis` is the style when it is highlighted, like being hovered by mouse, or highlighted via legend connect.""").tag(sync=True)
    
    emphasis = Dict(default_value=None, allow_none=True, help="""Map area style in highlighted state.""").tag(sync=True)
    
    select = Dict(default_value=None, allow_none=True, help="""Map area style in selected state.""").tag(sync=True)
    
    zlevel = Float(None, allow_none=True, help="""`zlevel` value of all graphical elements in .


`zlevel` is used to make layers with Canvas. Graphical elements with different `zlevel` values will be placed in different Canvases, which is a common optimization technique. We can put those frequently changed elements (like those with animations) to a separate `zlevel`. Notice that too many Canvases will increase memory cost, and should be used carefully on mobile phones to avoid crash.


Canvases with bigger `zlevel` will be placed on Canvases with smaller `zlevel`.""").tag(sync=True)
    
    z = Float(None, allow_none=True, help="""`z` value of all graphical elements in , which controls order of drawing graphical components. Components with smaller `z` values may be overwritten by those with larger `z` values.


`z` has a lower priority to `zlevel`, and will not create new Canvas.""").tag(sync=True)
    
    left = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between component and the left side of the container.


`left` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`; and it can also be `'left'`, `'center'`, or `'right'`.


If the `left` value is set to be `'left'`, `'center'`, or `'right'`, then the component will be aligned automatically based on position.""").tag(sync=True)
    
    top = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between component and the top side of the container.


`top` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`; and it can also be `'top'`, `'middle'`, or `'bottom'`.


If the `top` value is set to be `'top'`, `'middle'`, or `'bottom'`, then the component will be aligned automatically based on position.""").tag(sync=True)
    
    right = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between component and the right side of the container.


`right` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`.


Adaptive by default.""").tag(sync=True)
    
    bottom = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between component and the bottom side of the container.


`bottom` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`.


Adaptive by default.""").tag(sync=True)
    
    layoutCenter = Any(None, allow_none=True, help="""`layoutCenter` and `layoutSize` provides layout strategy other than `left/right/top/bottom/width/height`.


When using `left/right/top/bottom/width/height`, it is hard to put the map inside a box area with a fixed width-height ratio. In this case, `layoutCenter` attribute can be used to define the center position of map, and `layoutSize` can be used to define the size of map. For example:



```
layoutCenter: ['30%', '30%'],
// If width-height ratio is larger than 1, then width is set to be 100.
// Otherwise, height is set to be 100.
// This makes sure that it will not exceed the area of 100x100
layoutSize: 100

```

After setting these two values, `left/right/top/bottom/width/height` becomes invalid.""").tag(sync=True)
    
    layoutSize = Union([Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Size of map, see `layoutCenter` for more information. Percentage relative to screen width, and absolute pixel values are supported.""").tag(sync=True)
    
    geoIndex = Float(None, allow_none=True, help="""In default case, map series create exclusive `geo` component for themselves. But `geoIndex` can be used to specify an outer [geo component](#geo), which can be shared with other series like [pie](#series-pie). Moreover, the region color of the outer [geo component](#geo) can be controlled by the map series (via [visualMap](#visualMap)).


When `geoIndex` specified, [series-map.map](#series-map.map) other style configurations like [series-map.itemStyle](#series-map.itemStyle) will not work, but corresponding configurations in [geo component](#geo) will be used.


For example:""").tag(sync=True)
    
    mapValueCalculation = Unicode(None, allow_none=True, help="""Value of multiple series with the same [map type](#series-map.map) can use this option to get a statistical value.


Supported statistical methods:


* `'sum'`
* `'average'`
* `'max'`
* `'min'`""").tag(sync=True)
    
    showLegendSymbol = Bool(None, allow_none=True, help="""Show the symbol in related area (dot of series symbol). Available when [legend](#legend) component exists.""").tag(sync=True)
    
    seriesLayoutBy = Unicode(None, allow_none=True, help="""When [dataset](#dataset) is used, `seriesLayoutBy` specifies whether the column or the row of `dataset` is mapped to the series, namely, the series is "layout" on columns or rows. Optional values:


* 'column': by default, the columns of `dataset` are mapped the series. In this case, each column represents a dimension.
* 'row'：the rows of `dataset` are mapped to the series. In this case, each row represents a dimension.


Check this [example](https://echarts.apache.org/examples/en/editor.html?c=dataset-series-layout-by).""").tag(sync=True)
    
    datasetIndex = Float(None, allow_none=True, help="""If [series.data](#series.data) is not specified, and [dataset](#dataset) exists, the series will use `dataset`. `datasetIndex` specifies which dataset will be used.""").tag(sync=True)
    
    dataGroupId = Unicode(None, allow_none=True, help="""A group ID assigned to all data items in the series.


This option has a lower priority than `groupId`, which means when `groupId` is specified for a certain data item the `dataGroupId` will be simply ignored for that data item. For more information, please see `series.data.groupId`.""").tag(sync=True)
    
    labelLayout = Union([Any(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""> Since `v5.0.0`


Unified layout configuration of labels.


It provide a chance to adjust the labels' `(x, y)` position, alignment based on the original layout each series provides.


This option can be a callback with following parameters.



```
// corresponding index of data
dataIndex: number
// corresponding type of data. Only available in graph, in which it can be 'node' or 'edge'
dataType?: string
// corresponding index of series
seriesIndex: number
// Displayed text of label.
text: string
// Bounding rectangle of label.
labelRect: {x: number, y: number, width: number, height: number}
// Horizontal alignment of label.
align: 'left' | 'center' | 'right'
// Vertical alignment of label.
verticalAlign: 'top' | 'middle' | 'bottom'
// Bounding rectangle of the element corresponding to.
rect: {x: number, y: number, width: number, height: number}
// Default points array of labelLine. Currently only provided in pie and funnel series.
// It's null in other series.
labelLinePoints?: number[][]

```

**Example:**


Align the labels on the right. Left 10px margin to the edge.



```
labelLayout(params) {
    return {
        x: params.rect.x + 10,
        y: params.rect.y + params.rect.height / 2,
        verticalAlign: 'middle',
        align: 'left'
    }
}

```

Set the text size based on the size of element bounding rectangle.



```

labelLayout(params) {
    return {
        fontSize: Math.max(params.rect.width / 10, 5)
    };
}

```""").tag(sync=True)
    
    labelLine = Dict(default_value=None, allow_none=True, help="""> Since `v5.0.0`


Configuration of label guide line.""").tag(sync=True)
    
    data = Any(None, allow_none=True, help="""Data array of map series, which can be a single data value, like:



```
[12, 34, 56, 10, 23]

```

Or, if need extra dimensions for components like [visualMap](#visualMap) to map to graphic attributes like color, it can also be in the form of array. For example:



```
[[12, 14], [34, 50], [56, 30], [10, 15], [23, 10]]

```

In this case, we can assign the second value in each array item to [visualMap](#visualMap) component.


More likely, we need to assign name to each data item, in which case each item should be an object:



```
[{
    // name of date item
    name: 'data1',
    // value of date item is 8
    value: 10
}, {
    name: 'data2',
    value: 20
}]

```

Each data item can be further customized:



```
[{
    name: 'data1',
    value: 10
}, {
    // name of data item
    name: 'data2',
    value : 56,
    // user-defined label format that only useful for this data item
    label: {},
    // user-defined special itemStyle that only useful for this data item
    itemStyle:{}
}]

```""").tag(sync=True)
    
    markPoint = Dict(default_value=None, allow_none=True, help="""Mark point in a chart.""").tag(sync=True)
    
    markLine = Dict(default_value=None, allow_none=True, help="""Use a line in the chart to illustrate.""").tag(sync=True)
    
    markArea = Dict(default_value=None, allow_none=True, help="""Used to mark an area in chart. For example, mark a time interval.""").tag(sync=True)
    
    silent = Bool(None, allow_none=True, help="""Whether to ignore mouse events. Default value is false, for triggering and responding to mouse events.""").tag(sync=True)
    
    universalTransition = Any(None, allow_none=True, help="""> Since `v5.2.0`


Configuration related to universal transition animation.


Universal Transition provides the ability to morph between any series. With this feature enabled, each time `setOption`, transitions between series with the same `id` will be automatically associated with each other.


One-to-many or many-to-one animations such as drill-down, aggregation, etc. can also be achieved by specifying data items' `groupId` and `childGroupId`.


This can be enabled directly by configuring `universalTransition: true` in the series. It is also possible to provide an object for more detailed configuration.""").tag(sync=True)
    
    tooltip = Dict(default_value=None, allow_none=True, help="""tooltip settings in this series.""").tag(sync=True)
    
        



from traitlets import (
    Unicode, Dict, Float, Union, Any,
)
from ..basewidget import BaseWidget


class Polar(BaseWidget):
    """ 
:warning: **Autogenerated class**

Polar coordinate can be used in scatter and line chart. Every polar coordinate has an [angleAxis](#angleAxis) and a [radiusAxis](#radiusAxis).


**For example:** 




    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("PolarModel").tag(sync=True)
    
    id = Unicode(None, allow_none=True, help="""Component ID, not specified by default. If specified, it can be used to refer the component in option or API.""").tag(sync=True)
    
    zlevel = Float(None, allow_none=True, help="""`zlevel` value of all graphical elements in .


`zlevel` is used to make layers with Canvas. Graphical elements with different `zlevel` values will be placed in different Canvases, which is a common optimization technique. We can put those frequently changed elements (like those with animations) to a separate `zlevel`. Notice that too many Canvases will increase memory cost, and should be used carefully on mobile phones to avoid crash.


Canvases with bigger `zlevel` will be placed on Canvases with smaller `zlevel`.""").tag(sync=True)
    
    z = Float(None, allow_none=True, help="""`z` value of all graphical elements in , which controls order of drawing graphical components. Components with smaller `z` values may be overwritten by those with larger `z` values.


`z` has a lower priority to `zlevel`, and will not create new Canvas.""").tag(sync=True)
    
    center = Any(None, allow_none=True, help="""Center position of Polar coordinate, the first of which is the horizontal position, and the second is the vertical position.


Percentage is supported. When set in percentage, the item is relative to the container width, and the second item to the height.


**Example:** 



```
// Set to absolute pixel values
center: [400, 300]
// Set to relative percent
center: ['50%', '50%']

```""").tag(sync=True)
    
    radius = Union([Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Radius of Polar coordinate. Value can be:


* `number`: Specify outside radius directly.
* `string`: For example, `'20%'`, means that the outside radius is 20% of the viewport size (the little one between width and height of the chart container).


* `Array.<number|string>`: The first item specifies the inside radius, and the second item specifies the outside radius. Each item follows the definitions above.""").tag(sync=True)
    
    tooltip = Dict(default_value=None, allow_none=True, help="""tooltip settings in the coordinate system component.


**General Introduction:**


tooltip can be configured on different places:


* Configured on global: [tooltip](#tooltip)
* Configured in a coordinate system: [grid.tooltip](#grid.tooltip), [polar.tooltip](#polar.tooltip), [single.tooltip](#single.tooltip)
* Configured in a series: [series.tooltip](#series.tooltip)
* Configured in each item of `series.data`: [series.data.tooltip](#series.data.tooltip)""").tag(sync=True)
    
        



from traitlets import (
    Unicode, Dict, Float, Bool, Union, Any,
)
from ..basewidget import BaseWidget


class SeriesRadar(BaseWidget):
    """ 
:warning: **Autogenerated class**

**radar chart**


Radar chart is mainly used to show multi-variable data, such as the analysis of a football player's varied attributes. It relies [radar](#radar) component.


Here is the example of AQI data which is presented in radar chart.




    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("SeriesRadarModel").tag(sync=True)
    
    type = Unicode("radar", allow_none=True, ).tag(sync=True)
    
    id = Unicode(None, allow_none=True, help="""Component ID, not specified by default. If specified, it can be used to refer the component in option or API.""").tag(sync=True)
    
    name = Unicode(None, allow_none=True, help="""Series name used for displaying in [tooltip](#tooltip) and filtering with [legend](#legend), or updating data and configuration with `setOption`.""").tag(sync=True)
    
    colorBy = Unicode(None, allow_none=True, help="""> Since `v5.2.0`


The policy to take color from [option.color](#color). Valid values:


* `'series'`: assigns the colors in the palette by series, so that all data in the same series are in the same color;
* `'data'`: assigns colors in the palette according to data items, with each data item using a different color.""").tag(sync=True)
    
    radarIndex = Float(None, allow_none=True, help="""Index of [radar](#radar) component that radar chart uses.""").tag(sync=True)
    
    symbol = Union([Unicode(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Symbol of .


Icon types provided by ECharts includes


`'circle'`, `'rect'`, `'roundRect'`, `'triangle'`, `'diamond'`, `'pin'`, `'arrow'`, `'none'`


It can be set to an image with `'image://url'` , in which URL is the link to an image, or `dataURI` of an image.


An image URL example:



```
'image://http://example.website/a/b.png'

```
A `dataURI` example:



```
'image://data:image/gif;base64,R0lGODlhEAAQAMQAAORHHOVSKudfOulrSOp3WOyDZu6QdvCchPGolfO0o/XBs/fNwfjZ0frl3/zy7////wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAkAABAALAAAAAAQABAAAAVVICSOZGlCQAosJ6mu7fiyZeKqNKToQGDsM8hBADgUXoGAiqhSvp5QAnQKGIgUhwFUYLCVDFCrKUE1lBavAViFIDlTImbKC5Gm2hB0SlBCBMQiB0UjIQA7'

```
Icons can be set to arbitrary vector path via `'path://'` in ECharts. As compared with a raster image, vector paths prevent jagging and blurring when scaled, and have better control over changing colors. The size of the vector icon will be adapted automatically. Refer to [SVG PathData](http://www.w3.org/TR/SVG/paths.html#PathData) for more information about the format of the path. You may export vector paths from tools like Adobe 


For example:



```
'path://M30.9,53.2C16.8,53.2,5.3,41.7,5.3,27.6S16.8,2,30.9,2C45,2,56.4,13.5,56.4,27.6S45,53.2,30.9,53.2z M30.9,3.5C17.6,3.5,6.8,14.4,6.8,27.6c0,13.3,10.8,24.1,24.101,24.1C44.2,51.7,55,40.9,55,27.6C54.9,14.4,44.1,3.5,30.9,3.5z M36.9,35.8c0,0.601-0.4,1-0.9,1h-1.3c-0.5,0-0.9-0.399-0.9-1V19.5c0-0.6,0.4-1,0.9-1H36c0.5,0,0.9,0.4,0.9,1V35.8z M27.8,35.8 c0,0.601-0.4,1-0.9,1h-1.3c-0.5,0-0.9-0.399-0.9-1V19.5c0-0.6,0.4-1,0.9-1H27c0.5,0,0.9,0.4,0.9,1L27.8,35.8L27.8,35.8z'

```
If symbols needs to be different, you can set with callback function in the following format:



```
(value: Array|number, params: Object) => string

```

The first parameter `value` is the value in [data](#series-radar.data), and the second parameter `params` is the rest parameters of data item.""").tag(sync=True)
    
    symbolSize = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""symbol size. It can be set to single numbers like `10`, or use an array to represent width and height. For example, `[20, 10]` means symbol width is `20`, and height is`10`.


If size of symbols needs to be different, you can set with callback function in the following format:



```
(value: Array|number, params: Object) => number|Array

```

The first parameter `value` is the value in [data](#series-radar.data), and the second parameter `params` is the rest parameters of data item.""").tag(sync=True)
    
    symbolRotate = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Rotate degree of symbol. The negative value represents clockwise. Note that when `symbol` is set to be `'arrow'` in `markLine`, `symbolRotate` value will be ignored, and compulsively use tangent angle.


If rotation of symbols needs to be different, you can set with callback function in the following format:



```
(value: Array|number, params: Object) => number

```

The first parameter `value` is the value in [data](#series-radar.data), and the second parameter `params` is the rest parameters of data item.



> Callback is supported since 4.8.0 .""").tag(sync=True)
    
    symbolKeepAspect = Bool(None, allow_none=True, help="""Whether to keep aspect for symbols in the form of `path://`.""").tag(sync=True)
    
    symbolOffset = Any(None, allow_none=True, help="""Offset of symbol relative to original position. By default, symbol will be put in the center position of data. But if symbol is from user-defined vector path or image, you may not expect symbol to be in center. In this case, you may use this attribute to set offset to default position. It can be in absolute pixel value, or in relative percentage value.


For example, `[0, '-50%']` means to move upside side position of symbol height. It can be used to make the arrow in the bottom to be at data position when symbol is pin.""").tag(sync=True)
    
    label = Dict(default_value=None, allow_none=True, help="""Text label of , to explain some data information about graphic item like value, name and so on. `label` is placed under `itemStyle` in ECharts 2.x. In ECharts 3, to make the configuration structure flatter, `label`is taken to be at the same level with `itemStyle`, and has `emphasis` as `itemStyle` does.""").tag(sync=True)
    
    labelLayout = Union([Any(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""> Since `v5.0.0`


Unified layout configuration of labels.


It provide a chance to adjust the labels' `(x, y)` position, alignment based on the original layout each series provides.


This option can be a callback with following parameters.



```
// corresponding index of data
dataIndex: number
// corresponding type of data. Only available in graph, in which it can be 'node' or 'edge'
dataType?: string
// corresponding index of series
seriesIndex: number
// Displayed text of label.
text: string
// Bounding rectangle of label.
labelRect: {x: number, y: number, width: number, height: number}
// Horizontal alignment of label.
align: 'left' | 'center' | 'right'
// Vertical alignment of label.
verticalAlign: 'top' | 'middle' | 'bottom'
// Bounding rectangle of the element corresponding to.
rect: {x: number, y: number, width: number, height: number}
// Default points array of labelLine. Currently only provided in pie and funnel series.
// It's null in other series.
labelLinePoints?: number[][]

```

**Example:**


Align the labels on the right. Left 10px margin to the edge.



```
labelLayout(params) {
    return {
        x: params.rect.x + 10,
        y: params.rect.y + params.rect.height / 2,
        verticalAlign: 'middle',
        align: 'left'
    }
}

```

Set the text size based on the size of element bounding rectangle.



```

labelLayout(params) {
    return {
        fontSize: Math.max(params.rect.width / 10, 5)
    };
}

```""").tag(sync=True)
    
    itemStyle = Dict(default_value=None, allow_none=True, help="""Item style of the inflection point of the lines.""").tag(sync=True)
    
    lineStyle = Dict(default_value=None, allow_none=True, help="""Line style.""").tag(sync=True)
    
    areaStyle = Dict(default_value=None, allow_none=True, help="""Area filling style.""").tag(sync=True)
    
    emphasis = Dict(default_value=None, allow_none=True, help="""Configurations of emphasis state.""").tag(sync=True)
    
    blur = Dict(default_value=None, allow_none=True, help="""> Since `v5.0.0`


Configurations of blur state. Available when [emphasis.focus](#series-radar.emphasis.focus) is set.""").tag(sync=True)
    
    select = Dict(default_value=None, allow_none=True, help="""> Since `v5.0.0`


Configurations of select state. Available when [selectedMode](#series-radar.selectedMode) is set.""").tag(sync=True)
    
    selectedMode = Union([Bool(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""> Since `v5.0.0`


Selected mode. It is enabled by default, and you may set it to be `false` to disable it.


Besides, it can be set to `'single'`, `'multiple'` or `'series'`, for single selection, multiple selections and whole series selection.



> `'series'` is supported since v5.3.0""").tag(sync=True)
    
    dataGroupId = Unicode(None, allow_none=True, help="""A group ID assigned to all data items in the series.


This option has a lower priority than `groupId`, which means when `groupId` is specified for a certain data item the `dataGroupId` will be simply ignored for that data item. For more information, please see `series.data.groupId`.""").tag(sync=True)
    
    data = Any(None, allow_none=True, help="""The data in radar chart is multi-variable (dimension). Here is an example:



```
data : [
    {
        value : [4300, 10000, 28000, 35000, 50000, 19000],
        name : 'Allocated Budget'
    },
    {
        value : [5000, 14000, 28000, 31000, 42000, 21000],
        name : 'Actual Spending'
    }
]

```

Among them, `value` item array contains data that is corresponding to [radar.indicator](#radar.indicator).""").tag(sync=True)
    
    zlevel = Float(None, allow_none=True, help="""`zlevel` value of all graphical elements in Radar.


`zlevel` is used to make layers with Canvas. Graphical elements with different `zlevel` values will be placed in different Canvases, which is a common optimization technique. We can put those frequently changed elements (like those with animations) to a separate `zlevel`. Notice that too many Canvases will increase memory cost, and should be used carefully on mobile phones to avoid crash.


Canvases with bigger `zlevel` will be placed on Canvases with smaller `zlevel`.""").tag(sync=True)
    
    z = Float(None, allow_none=True, help="""`z` value of all graphical elements in Radar, which controls order of drawing graphical components. Components with smaller `z` values may be overwritten by those with larger `z` values.


`z` has a lower priority to `zlevel`, and will not create new Canvas.""").tag(sync=True)
    
    silent = Bool(None, allow_none=True, help="""Whether to ignore mouse events. Default value is false, for triggering and responding to mouse events.""").tag(sync=True)
    
    animation = Bool(None, allow_none=True, help="""Whether to enable animation.""").tag(sync=True)
    
    animationThreshold = Float(None, allow_none=True, help="""Whether to set graphic number threshold to animation. Animation will be disabled when graphic number is larger than threshold.""").tag(sync=True)
    
    animationDuration = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Duration of the first animation, which supports callback function for different data to have different animation effect:



```
animationDuration: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```""").tag(sync=True)
    
    animationEasing = Unicode(None, allow_none=True, help="""Easing method used for the first animation. Varied easing effects can be found at [easing effect example](https://echarts.apache.org/examples/en/editor.html?c=line-easing).""").tag(sync=True)
    
    animationDelay = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Delay before updating the first animation, which supports callback function for different data to have different animation effect.


For example:



```
animationDelay: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```

See [this example](https://echarts.apache.org/examples/en/editor.html?c=bar-animation-delay) for more information.""").tag(sync=True)
    
    animationDurationUpdate = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Time for animation to complete, which supports callback function for different data to have different animation effect:



```
animationDurationUpdate: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```""").tag(sync=True)
    
    animationEasingUpdate = Unicode(None, allow_none=True, help="""Easing method used for animation.""").tag(sync=True)
    
    animationDelayUpdate = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Delay before updating animation, which supports callback function for different data to have different animation effects.


For example:



```
animationDelayUpdate: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```

See [this example](https://echarts.apache.org/examples/en/editor.html?c=bar-animation-delay) for more information.""").tag(sync=True)
    
    universalTransition = Any(None, allow_none=True, help="""> Since `v5.2.0`


Configuration related to universal transition animation.


Universal Transition provides the ability to morph between any series. With this feature enabled, each time `setOption`, transitions between series with the same `id` will be automatically associated with each other.


One-to-many or many-to-one animations such as drill-down, aggregation, etc. can also be achieved by specifying data items' `groupId` and `childGroupId`.


This can be enabled directly by configuring `universalTransition: true` in the series. It is also possible to provide an object for more detailed configuration.""").tag(sync=True)
    
    tooltip = Dict(default_value=None, allow_none=True, help="""tooltip settings in this series.""").tag(sync=True)
    
        



from copy import deepcopy
from pathlib import Path
from typing import Dict, List, Tuple

import markdownify
from jinja2 import Environment, FileSystemLoader, select_autoescape

from .tools import (
    capitalize,
    convert_object_type,
    is_array_type,
    is_object_type,
)
import re
from .traitfactory import generate_trait

pattern = re.compile(r"(?<!^)(?=[A-Z])")


class CodeGen:
    def __init__(self, base_name: str, schema: dict, template_path: Path) -> None:
        self._schema = schema
        self.template_env = Environment(
            loader=FileSystemLoader(template_path), autoescape=select_autoescape()
        )
        self._base_name = base_name
        self._ts_files: List[str] = []
        self._init_content = {}

    def _generate_object_type(
        self,
        trait_name: str,
        trait_value: Dict,
        output: Path,
        ts_output: Path,
        type_value=None,
    ) -> str:
        trait_copy = deepcopy(trait_value)
        trait_copy["type"] = convert_object_type(trait_copy["type"])

        desc: str = trait_value.pop("description", "")
        traits = []
        defaults = []
        cls_name = capitalize(trait_name)

        properties: Dict = trait_value.get("properties", {})

        for name, props in properties.items():
            traits.append(generate_trait(name, props, type_value=type_value))

        self._write_to_py_file(cls_name, desc, traits, [], output)
        self._init_content[cls_name.lower()] = cls_name
        self._write_to_ts_file(cls_name=cls_name, defaults=defaults, output=ts_output)
        return cls_name

    def _generate_array_type(
        self,
        trait_name: str,
        trait_value: Dict[str, Dict],
        output: Path,
        ts_output: Path,
    ) -> Tuple[List[str], List[str]]:
        cls_name = capitalize(trait_name)
        items = trait_value.get("items", {})
        desc: str = trait_value.pop("description", "")

        imports = []
        traits = []
        defaults = []
        out_dir = output / f"{trait_name.lower()}items"
        out_dir.mkdir(exist_ok=True)
        init_path = out_dir / "__init__.py"
        if not init_path.exists():
            open(init_path, "a").close()

        ts_out_dir = ts_output / f"{trait_name.lower()}items"
        ts_out_dir.mkdir(exist_ok=True)
        if "anyOf" in items:
            # List of trais
            trait_item = ""
            for item_idx, list_item in enumerate(items["anyOf"]):
                if is_object_type(list_item):
                    type_value = (
                        list_item.get("properties", {})
                        .get("type", {})
                        .get("default", None)
                    )

                    if type_value:
                        type_value = type_value.replace("'", "").replace('"', "")
                        if trait_name == "series" and type_value in [
                            "radar",
                            "parallel",
                        ]:
                            list_item_cls = f"{trait_name}{capitalize(type_value)}"
                        else:
                            list_item_cls = f"{capitalize(type_value)}"

                    else:
                        list_item_cls = f"{trait_name}Item{item_idx}"
                    item_cls = self._generate_object_type(
                        list_item_cls,
                        list_item,
                        out_dir,
                        ts_out_dir,
                        type_value=type_value,
                    )
                    trait_item += f"Instance({item_cls}, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),"
                    imports.append(
                        {
                            "module": f".{trait_name.lower()}items.{item_cls.lower()}",
                            "import_name": item_cls,
                        }
                    )
            code = f"{trait_name} = List(trait=Union([{trait_item}]), allow_none=True, default_value=None).tag(sync=True, **widget_serialization)"
            defaults.append({"name": trait_name, "value": []})
            traits.append(code)

        elif "type" in items:
            # One trait
            if is_object_type(items):
                import_name = self._generate_object_type(
                    f"{trait_name}Item", items, out_dir, ts_out_dir
                )
                imports.append(
                    {
                        "module": f".{trait_name.lower()}items.{import_name.lower()}",
                        "import_name": import_name,
                    }
                )
                code = f"{trait_name} = List(trait=Instance({import_name}), default_value=None, allow_none=True).tag(sync=True, **widget_serialization)"
                traits.append(code)

        self._write_to_py_file(cls_name, desc, traits, imports, output)
        if cls_name == "Series":
            serializers_attrs = ["series"]
        else:
            serializers_attrs = []
        self._write_to_ts_file(
            cls_name=cls_name,
            defaults=defaults,
            output=ts_output,
            serializers_attrs=serializers_attrs,
        )
        return traits, imports

    def _write_to_py_file(
        self,
        cls_name: str,
        desc: str,
        traits: List[str],
        imports: List[str],
        output: Path,
    ):
        template = self.template_env.get_template("class.py.jinja")
        class_name_fixed = capitalize(cls_name)
        md_desc = markdownify.markdownify(f"{desc}", bullets="-")
        resources = dict(
            class_name=class_name_fixed,
            desc=md_desc,
            # desc=desc,
            traits=traits,
            imports=imports,
            model_name=f"{class_name_fixed}Model",
        )

        content = template.render(**resources)
        file_path = output / f"{cls_name.lower()}.py"
        with open(file_path, "w") as f:
            f.write(content)

    def _write_to_ts_file(
        self,
        cls_name: str,
        defaults: List[Dict],
        output: Path,
        serializers_attrs: List[str] = [],
    ):
        template = self.template_env.get_template("class.ts.jinja")
        class_name_fixed = capitalize(cls_name)
        resources = dict(
            class_name=class_name_fixed,
            model_name=f"{class_name_fixed}Model",
            defaults=defaults,
            len=len,
            serializers_attrs=serializers_attrs,
        )

        content = template.render(**resources)
        ts_file_name = output / f"{cls_name.lower()}.ts"
        with open(ts_file_name, "w") as f:
            f.write(content)
        self._ts_files.append(str(ts_file_name))

    def _write_ts_index(self, ts_output: Path):
        with open(ts_output / "index.ts", "w") as f:
            for file in self._ts_files:
                export_path = file.replace(".ts", "").replace(str(ts_output), ".")
                f.writelines(f"export * from '{export_path}';\n")

        with open(ts_output / "version.ts", "w") as f:
            f.write(
                "import { MODULE_NAME, MODULE_VERSION } from '../version';\nexport { MODULE_NAME, MODULE_VERSION };\n"
            )
        with open(ts_output / "types.ts", "w") as f:
            f.write(
                "import { IUpdateManager } from '../types';\nexport { IUpdateManager };\n"
            )

    def _write_init(self, class_dir: Path):

        init_path = class_dir / "__init__.py"
        content = ["from .option import Option"]
        for path in class_dir.glob("*.py"):
            file_name = path.name.split(".")[0]
            if file_name in self._init_content:
                content.append(
                    f"from .{file_name} import {self._init_content[file_name]}"
                )

        content_str = "\n".join(content)
        with open(init_path, "w") as f:
            f.write(content_str)

    def generate(self, output: Path, ts_output: Path):
        imports = []
        traits = []
        defaults = []
        option_props = []
        for prop, prop_value in self._schema["properties"].items():
            cls_name = None
            list_traits = None
            list_imports = None
            if is_object_type(prop_value):
                cls_name = self._generate_object_type(
                    prop, prop_value, output, ts_output
                )

            elif is_array_type(prop_value):
                if "items" not in prop_value:
                    traits.append(
                        f"{prop} = List(trait=Any, default_value=None, allow_none=True).tag(sync=True)"
                    )
                    defaults.append({"name": prop, "value": []})
                else:
                    list_traits, list_imports = self._generate_array_type(
                        prop, prop_value, output, ts_output
                    )
            else:
                traits.append(generate_trait(prop, prop_value))
            if cls_name is not None:
                imports.append(
                    {"module": f".{cls_name.lower()}", "import_name": cls_name}
                )
                traits.append(
                    f"{prop} = Instance({cls_name}, kw=None, args=None,allow_none=True).tag(sync=True, **widget_serialization)"
                )
                defaults.append({"name": prop, "value": {}})
                option_props.append(prop)
            elif list_traits is not None:
                traits.extend(list_traits)
                if list_imports is not None:
                    imports.extend(list_imports)

        self._write_to_py_file(
            "Option", desc="", traits=traits, imports=imports, output=output
        )

        option_props.append("series")
        defaults.append({"name": "series", "value": []})
        self._write_to_ts_file(
            cls_name="Option",
            defaults=defaults,
            output=ts_output,
            serializers_attrs=option_props,
        )
        # self._ts_files.append(str(ts_output / "option.ts"))
        self._write_ts_index(ts_output)
        self._write_init(output)



from traitlets import (
    Unicode, Dict, Float, Bool, Union, Any,
)
from ..basewidget import BaseWidget


class Grid(BaseWidget):
    """ 
:warning: **Autogenerated class**

Drawing grid in rectangular coordinate. In a single grid, at most two X and Y axes each is allowed. [Line chart](#series-line), [bar chart](#series-bar), and [scatter chart (bubble chart)](#series-scatter) can be drawn in grid.


In ECharts 2.x, there could only be one single grid component at most in a single echarts instance. But in ECharts 3, there is no limitation.


**Following is an example of Anscombe Quartet:**




    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("GridModel").tag(sync=True)
    
    id = Unicode(None, allow_none=True, help="""Component ID, not specified by default. If specified, it can be used to refer the component in option or API.""").tag(sync=True)
    
    show = Bool(None, allow_none=True, help="""Whether to show the grid in rectangular coordinate.""").tag(sync=True)
    
    zlevel = Float(None, allow_none=True, help="""`zlevel` value of all graphical elements in .


`zlevel` is used to make layers with Canvas. Graphical elements with different `zlevel` values will be placed in different Canvases, which is a common optimization technique. We can put those frequently changed elements (like those with animations) to a separate `zlevel`. Notice that too many Canvases will increase memory cost, and should be used carefully on mobile phones to avoid crash.


Canvases with bigger `zlevel` will be placed on Canvases with smaller `zlevel`.""").tag(sync=True)
    
    z = Float(None, allow_none=True, help="""`z` value of all graphical elements in , which controls order of drawing graphical components. Components with smaller `z` values may be overwritten by those with larger `z` values.


`z` has a lower priority to `zlevel`, and will not create new Canvas.""").tag(sync=True)
    
    left = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between grid component and the left side of the container.


`left` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`; and it can also be `'left'`, `'center'`, or `'right'`.


If the `left` value is set to be `'left'`, `'center'`, or `'right'`, then the component will be aligned automatically based on position.""").tag(sync=True)
    
    top = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between grid component and the top side of the container.


`top` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`; and it can also be `'top'`, `'middle'`, or `'bottom'`.


If the `top` value is set to be `'top'`, `'middle'`, or `'bottom'`, then the component will be aligned automatically based on position.""").tag(sync=True)
    
    right = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between grid component and the right side of the container.


`right` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`.""").tag(sync=True)
    
    bottom = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between grid component and the bottom side of the container.


`bottom` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`.""").tag(sync=True)
    
    width = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Width of grid component. Adaptive by default.""").tag(sync=True)
    
    height = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Height of grid component. Adaptive by default.""").tag(sync=True)
    
    containLabel = Bool(None, allow_none=True, help="""Whether the grid region contains [axis tick label](#yAxis.axisLabel) of axis.


* When containLabel is `false`:
	+ `grid.left` `grid.right` `grid.top` `grid.bottom` `grid.width` `grid.height` decide the location and size of the rectangle that is made of by xAxis and yAxis.
	+ Setting to `false` will help when multiple grids need to be aligned at their axes.
* When containLabel is `true`:
	+ `grid.left` `grid.right` `grid.top` `grid.bottom` `grid.width` `grid.height` decide the location and size of the rectangle that contains the axes and the labels of the axes.
	+ Setting to `true` will help when the length of axis labels is dynamic and is hard to approximate. This will avoid labels from overflowing the container or overlapping other components.""").tag(sync=True)
    
    backgroundColor = Any(None, allow_none=True, help="""Background color of grid, which is transparent by default.



> Color can be represented in RGB, for example `'rgb(128, 128, 128)'`. RGBA can be used when you need alpha channel, for example `'rgba(128, 128, 128, 0.5)'`. You may also use hexadecimal format, for example `'#ccc'`.


**Attention**: Works only if `show: true` is set.""").tag(sync=True)
    
    borderColor = Any(None, allow_none=True, help="""Border color of grid. Support the same color format as backgroundColor.


**Attention**: Works only if `show: true` is set.""").tag(sync=True)
    
    borderWidth = Float(None, allow_none=True, help="""Border width of grid.


**Attention**: Works only if `show: true` is set.""").tag(sync=True)
    
    shadowBlur = Float(None, allow_none=True, help="""Size of shadow blur. This attribute should be used along with `shadowColor`,`shadowOffsetX`, `shadowOffsetY` to set shadow to component.


For example:



```
{
    shadowColor: 'rgba(0, 0, 0, 0.5)',
    shadowBlur: 10
}

```

**Attention**: This property works only if `show: true` is configured and `backgroundColor` is defined other than `transparent`.""").tag(sync=True)
    
    shadowColor = Any(None, allow_none=True, help="""Shadow color. Support same format as `color`.


**Attention**: This property works only if `show: true` configured.""").tag(sync=True)
    
    shadowOffsetX = Float(None, allow_none=True, help="""Offset distance on the horizontal direction of shadow.


**Attention**: This property works only if `show: true` configured.""").tag(sync=True)
    
    shadowOffsetY = Float(None, allow_none=True, help="""Offset distance on the vertical direction of shadow.


**Attention**: This property works only if `show: true` configured.""").tag(sync=True)
    
    tooltip = Dict(default_value=None, allow_none=True, help="""tooltip settings in the coordinate system component.


**General Introduction:**


tooltip can be configured on different places:


* Configured on global: [tooltip](#tooltip)
* Configured in a coordinate system: [grid.tooltip](#grid.tooltip), [polar.tooltip](#polar.tooltip), [single.tooltip](#single.tooltip)
* Configured in a series: [series.tooltip](#series.tooltip)
* Configured in each item of `series.data`: [series.data.tooltip](#series.data.tooltip)""").tag(sync=True)
    
        



from traitlets import (
    Unicode, Float, Union, Any,
)
from ..basewidget import BaseWidget


class TextStyle(BaseWidget):
    """ 
:warning: **Autogenerated class**

Global font style.



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("TextStyleModel").tag(sync=True)
    
    color = Any(None, allow_none=True, help="""text color.""").tag(sync=True)
    
    fontStyle = Unicode(None, allow_none=True, help="""font style.


Options are:


* `'normal'`
* `'italic'`
* `'oblique'`""").tag(sync=True)
    
    fontWeight = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""font thick weight.


Options are:


* `'normal'`
* `'bold'`
* `'bolder'`
* `'lighter'`
* 100 | 200 | 300 | 400...""").tag(sync=True)
    
    fontFamily = Unicode(None, allow_none=True, help="""font family.


Can also be 'serif' , 'monospace', ...""").tag(sync=True)
    
    fontSize = Float(None, allow_none=True, help="""font size.""").tag(sync=True)
    
    lineHeight = Float(None, allow_none=True, help="""Line height of the text fragment.


If `lineHeight` is not set in `rich`, `lineHeight` in parent level will be used. For example:



```
{
    lineHeight: 56,
    rich: {
        a: {
            // `lineHeight` is not set, then it will be 56
        }
    }
}

```""").tag(sync=True)
    
    width = Float(None, allow_none=True, help="""Width of text block.""").tag(sync=True)
    
    height = Float(None, allow_none=True, help="""Height of text block.""").tag(sync=True)
    
    textBorderColor = Any(None, allow_none=True, help="""Stroke color of the text.""").tag(sync=True)
    
    textBorderWidth = Float(None, allow_none=True, help="""Stroke line width of the text.""").tag(sync=True)
    
    textBorderType = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Stroke line type of the text.


Possible values are:


* `'solid'`
* `'dashed'`
* `'dotted'`


Since `v5.0.0`, it can also be a number or a number array to specify the [dash array](https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-dasharray) of the line. With 
`textBorderDashOffset`
, we can make the line style more flexible.


For example：



```
{

textBorderType: [5, 10],

textBorderDashOffset: 5
}

```""").tag(sync=True)
    
    textBorderDashOffset = Float(None, allow_none=True, help="""> Since `v5.0.0`


To set the line dash offset. With 
`textBorderType`
, we can make the line style more flexible.


Refer to MDN [lineDashOffset](https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/lineDashOffset) for more details.""").tag(sync=True)
    
    textShadowColor = Any(None, allow_none=True, help="""Shadow color of the text itself.""").tag(sync=True)
    
    textShadowBlur = Float(None, allow_none=True, help="""Shadow blue of the text itself.""").tag(sync=True)
    
    textShadowOffsetX = Float(None, allow_none=True, help="""Shadow X offset of the text itself.""").tag(sync=True)
    
    textShadowOffsetY = Float(None, allow_none=True, help="""Shadow Y offset of the text itself.""").tag(sync=True)
    
    overflow = Unicode(None, allow_none=True, help="""Determine how to display the text when it's overflow. Available when `width` is set.


* `'truncate'` Truncate the text and trailing with `ellipsis`.
* `'break'` Break by word
* `'breakAll'` Break by character.""").tag(sync=True)
    
    ellipsis = Unicode(None, allow_none=True, help="""Ellipsis to be displayed when `overflow` is set to `truncate`.


* `'truncate'` Truncate the overflow lines.""").tag(sync=True)
    
        



from traitlets import (
    Unicode, Union, Any,
)
from ..basewidget import BaseWidget


class Graphic(BaseWidget):
    """ 
:warning: **Autogenerated class**

`graphic` component enables creating graphic elements in ECharts.


Those graphic type are supported.


[image](#graphic.elements-image),
[text](#graphic.elements-text),
[circle](#graphic.elements-circle),
[sector](#graphic.elements-sector),
[ring](#graphic.elements-ring),
[polygon](#graphic.elements-polygon),
[polyline](#graphic.elements-polyline),
[rect](#graphic.elements-rect),
[line](#graphic.elements-line),
[bezierCurve](#graphic.elements-bezierCurve),
[arc](#graphic.elements-arc),
[group](#graphic.elements-group),


This example shows how to make a watermark and text block:



This example use hidden graphic elements to implement dragging:



**Graphic Component Configuration**


A simple way to define a graphic element:



```
myChart.setOption({
    ...,
    graphic: {
        type: 'image',
        ...
    }
});

```

Define multiple graphic elements:



```
myChart.setOption({
    ...,
    graphic: [
        { // A 'image' element.
            type: 'image',
            ...
        },
        { // A 'text' element, with id specified.
            type: 'text',
            id: 'text1',
            ...
        },
        { // A 'group' element, in which children can be defined.
            type: 'group',
            children: [
                {
                    type: 'rect',
                    id: 'rect1',
                    ...
                },
                {
                    type: 'image',
                    ...
                },
                ...
            ]
        }
        ...
    ]
});


```

How to remove or replace existing elements by `setOption`:



```
myChart.setOption({
    ...,
    graphic: [
        { // Remove the element 'text1' defined above.
            id: 'text1',
            $action: 'remove',
            ...
        },
        { // Replace the element 'rect1' to a new circle element.
          // Note, although in the sample above 'rect1' is a children of a group,
          // it is not necessary to consider level relationship when setOption
          // again if you use id to specify them.
            id: 'rect1',
            $action: 'replace',
            type: 'circle',
            ...
        }
    ]
});

```

Notice, when using `setOption` to modify existing elements, if id is not specified, new options will be mapped to existing elements by their order, which might bring unexpected result sometimes. So, generally, using id is recommended.


**Graphic Element Configuration**


Different types of graphic elements has their own configuration respectively, but they have these common configuration below:



```
{
    // id is used to specifying element when willing to update it.
    // id can be ignored if you do not need it.
    id: 'xxx',

    // Specify element type. Can not be ignored when define a element at the first time.
    type: 'image',

    // All of the properties below can be ignored and a default value will be assigned.

    // Specify the operation should be performed to the element when calling `setOption`.
    // Default value is 'merge', other values can be 'replace' or 'remove'.
    $action: 'replace',

    // These four properties is used to locating the element. Each property can be absolute
    // value (like 10, means 10 pixel) or percent (like '12%') or 'center'/'middle'.
    left: 10,
    // right: 10,
    top: 'center',
    // bottom: '10%',

    shape: {
        // Here are configurations for shape, like `x`, `y`, `cx`, `cy`, `width`,
        // `height`, `r`, `points`, ...
        // Note, if `left`/`right`/`top`/`bottom` has been set, `x`/`y`/`cx`/`cy`
        // do not work here.
    },

    style: {
        // Here are configurations for style of the element, like `fill`, `stroke`,
        // `lineWidth`, `shadowBlur`, ...
    },

    // z value of the elements.
    z: 10,
    // Whether response to mouse events / touch events.
    silent: true,
    // Whether the element is visible.
    invisible: false,
    // Used to specify whether the entire transformed element (containing children if is group)
    // is confined in its container. Optional values: 'raw', 'all'.
    bounding: 'raw',
    // Can be dragged or not.
    draggable: false,
    // Event handler, can also be onmousemove, ondrag, ... (listed below)
    onclick: function () {...}
}

```

**Event Handlers of Graphic Element**


These events are supported:
`onclick`, `onmouseover`, `onmouseout`, `onmousemove`, `onmousewheel`, `onmousedown`, `onmouseup`, `ondrag`, `ondragstart`, `ondragend`, `ondragenter`, `ondragleave`, `ondragover`, `ondrop`.


**Hierarchy of Graphic Elements**


Only `group` element has children, which enable a group of elements to be positioned and transformed together.


**Shape Configuration of Graphic Element**


Elements with different types have different shape setting respectively. For example:



```
{
    type: 'rect',
    shape: {
        x: 10,
        y: 10,
        width: 100,
        height: 200
    }
},
{
    type: 'circle',
    shape: {
        cx: 20,
        cy: 30,
        r: 100
    }
},
{
    type: 'image',
    style: {
        image: 'http://example.website/a.png',
        x: 100,
        y: 200,
        width: 230,
        height: 400
    }
},
{
    type: 'text',
    style: {
        text: 'This text',
        x: 100,
        y: 200
    }

}

```

**Transforming and Absolutely Positioning of Graphic Element**


Element can be transformed (translation, rotation, scale). See [position](#graphic.elements.position), [rotation](#graphic.elements.rotation), [scale](#graphic.elements.scale), [origin](#graphic.elements.origin)


For example:



```
{
    type: 'rect', // or any other types.

    // Translation, using [0, 0] by default.
    position: [100, 200],

    // Scale, using [1, 1] by default.
    scale: [2, 4],

    // Rotation, using 0 by default. Negative value means rotating clockwise.
    rotation: Math.PI / 4,

    // Origin point of rotation and scale, using [0, 0] by default.
    origin: [10, 20],

    shape: {
        // ...
    }
}

```

Each element is transformed in the coordinate system of its parent, namely, transform of a element and its parent can be "stacked".


Transformation is performed by this order:


1. Translate [-el.origin[0], -el.origin[1]].
2. Scale according to el.scale.
3. Rotate according to el.rotation.
4. Translate back according to el.origin.
5. Translate according to el.position.


Namely, scaling and rotating firstly, and then translate. By this mechanism, translation does not affect origin of scale and rotation.


**Relatively Positioning**


In real application, size of a container is always not fixed. So mechanism of relative position is required. In `graphic` component, [left](#graphic.elements.left) / [right](#graphic.elements.right) / [top](#graphic.elements.top) / [bottom](#graphic.elements.bottom) / [width](#graphic.elements.width) / [height](#graphic.elements.height) are used to position element relatively.


For example:



```
{ // Position the image at the bottom center of its container.
    type: 'image',
    left: 'center', // Position at the center horizontally.
    bottom: '10%',  // Position beyond the bottom boundary 10%.
    style: {
        image: 'http://example.website/a.png',
        width: 45,
        height: 45
    }
},
{ // Position the entire rotated group at the right-bottom corner of its container.
    type: 'group',
    right: 0,  // Position at the right boundary.
    bottom: 0, // Position at the bottom boundary.
    rotation: Math.PI / 4,
    children: [
        {
            type: 'rect',
            left: 'center', // Position at horizontal center according to its parent.
            top: 'middle',  // Position at vertical center according to its parent.
            shape: {
                width: 190,
                height: 90
            },
            style: {
                fill: '#fff',
                stroke: '#999',
                lineWidth: 2,
                shadowBlur: 8,
                shadowOffsetX: 3,
                shadowOffsetY: 3,
                shadowColor: 'rgba(0,0,0,0.3)'
            }
        },
        {
            type: 'text',
            left: 'center', // Position at horizontal center according to its parent.
            top: 'middle',  // Position at vertical center according to its parent.
            style: {
                fill: '#777',
                text: [
                    'This is text',
                    'This is text',
                    'Print some text'
                ].join('\n'),
                font: '14px Microsoft YaHei'
            }
        }
    ]
}

```

Note, [bounding](graphic.elements.bounding) can be used to specify whether the entire transformed element (containing children if is group) is confined in its container.



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("GraphicModel").tag(sync=True)
    
    id = Unicode(None, allow_none=True, help="""Component ID, not specified by default. If specified, it can be used to refer the component in option or API.""").tag(sync=True)
    
    elements = Union([Any(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""A list of all graphic elements.


Note, the standard configuration of graphic component is:



```
{
    graphic: {
        elements: [
            {type: 'rect', ...}, {type: 'circle', ...}, ...
        ]
    }
}

```

But we always use short patterns for convenience:



```
{
    graphic: { // Declare only one graphic element.
        type: 'rect',
        ...
    }
}

```

Or:



```
{
    graphic: [ // Declare multiple graphic elements.
        {type: 'rect', ...}, {type: 'circle', ...}, ...
    ]
}

```""").tag(sync=True)
    
        



from traitlets import (
    Unicode, Dict, Float, Bool, Union, Any,
)
from ..basewidget import BaseWidget


class Brush(BaseWidget):
    """ 
:warning: **Autogenerated class**

`brush` is an area-selecting component, with which user can select part of data from a chart to display in detail, or do calculations with them.


  





---


**Brush type and triggering button**


Currently, supported `brush` types include: `scatter`, `bar`, `candlestick`. (Note that `parallel` contains brush function by itself, which is not provided by brush component.)


Click the button in `toolbox` to enable operations like *area selecting*, or *canceling selecting*.


  

Example of `horizontal brush`: (Click the button in `toolbox` to enable brushing.)



  

Example of `brush` in `bar` charts:



Button for `brush` can be assigned in [`toolbox`](#toolbox.feature.brush.type) or [`brush` configuration](#brush.toolbox).


The following types of brushes are supported: `rect`, `polygon`, `lineX`, `lineY`. See [brush.toolbox](#brush.toolbox) for more information.


`keep` button can be used to toggle a single or multiple selections.


- Only one select box is available in single selection mode, and the select-box can be removed by clicking on the blank area.
- Multiple select boxes are available in multiple selection mode, and the select-boxes cannot be removed by click on the blank area. Instead, you need to click the *clear* button.


  





---


**Relationship between brush-selecting and coordinates**


`brush` can be set to be *global*, or *belonging to a particular coordinate*.


**Global brushes**


Selecting is enabled for everywhere in ECharts's instance in this case. This is the default situation, when brush is not set to be global.


**Coordinate brushes**


Selecting is enabled only in the assigned coordinates in this case. Selecting-box will be altered according to scaling and translating of coordinate (see `roam` and `dataZoom`).


In practice, you may often find coordinate brush to be a more frequently made choice, particularly in `geo` charts.


By assigning [brush.geoIndex](#brush.geoIndex), or [brush.xAxisIndex](#brush.xAxisIndex), or [brush.yAxisIndex](#brush.yAxisIndex), brush selecting axes can be assigned, whose value can be:


- `'all'`: for all axes;
- `number`: like `0`, for a particular coordinate with that index;
- `Array`: like `[0, 4, 2]`, for coordinates with those indexes;
- `'none'`, or `null`, or `undefined`: for not assigning.


Example:



```
option = {
    geo: {
        ...
    },
    brush: {
        geoIndex: 'all', // brush selecting is enabled only in all geo charts above
        ...
    }
};

```

Example:



```
option = {
    grid: [
        {...}, // grid 0
        {...}  // grid 1
    ],
    xAxis: [
        {gridIndex: 1, ...}, // xAxis 0 for grid 1
        {gridIndex: 0, ...}  // xAxis 1 for grid 0
    ],
    yAxis: [
        {gridIndex: 1, ...}, // yAxis 0 for grid 1
        {gridIndex: 0, ...}  // yAxis 1 for grid 0
    ],
    brush: {
        xAxisIndex: [0, 1], // brush selecting is enabled only in coordinates with xAxisIndex as `0` or `1`
        ...
    }
};

```

  





---


**Control select-box with API**


`dispatchAction` can be used to render select-box programmatically. For example:



```
myChart.dispatchAction({
    type: 'brush',
    areas: [
        {
            geoIndex: 0,
            // Assign select-box type
            brushType: 'polygon',
            // Assign select-box shape
            coordRange: [[119.72,34.85],[119.68,34.85],[119.5,34.84],[119.19,34.77]]
        }
    ]
});

```

Please refer to [action.brush](api.html#action.brush) for more information.


  





---


**brushLink**


Links interaction between selected items in different series.


Following is an example of enabling selected effect for `scatter` and `parallel` charts once a scatter chart is selected.


`brushLink` is an array of `seriesIndex`es, which assigns the series that can be interacted. For example, it can be:


- `[3, 4, 5]` for interacting series with seriesIndex as `3`, `4`, or `5`;
- `'all'` for interacting all series;
- `'none'`, or `null`, or `undefined` for disabling `brushLink`.


**Attention**


`brushLink` is a mapping of `dataIndex`. So **`data` of every series with `brushLink` should be guaranteed to correspond to the other**.


Example:



```
option = {
    brush: {
        brushLink: [0, 1]
    },
    series: [
        {
            type: 'bar'
            data: [232,    4434,    545,      654]     // data has 4 items
        },
        {
            type: 'parallel',
            data: [[4, 5], [3, 5], [66, 33], [99, 66]] // data also has 4 items, which correspond to the data above
        }
    ]
};

```

Please refer to [brush.brushLink](#brush.brushLink).


  





---


**throttle / debounce**


By default, `brushSelected` is always triggered when selection-box is selected or moved, to tell the outside about the event.


But efficiency problems may occur when events are triggered too frequently, or the animation result may be affected. So brush components provides [brush.throttleType](#brush.throttleType) and [brush.throttleDelay](#brush.throttleDelay) to solve this problem.


Valid `throttleType` values can be:


- `'debounce'`: for triggering events only when the action has been stopped (no action after some duration). Time threshold can be assigned with [brush.throttleDelay](#brush.throttleDelay);
- `'fixRate'`: for triggering event with a certain frequency. The frequency can be assigned with [brush.throttleDelay](#brush.throttleDelay).


In this [example](https://echarts.apache.org/examples/en/view.html?c=scatter-map-brush&edit=1&reset=1), `debounce` is used to make sure the bar chart is updated only when the user has stopped action. In other cases, the animation result may not be so good.


  





---


**Visual configurations of selected and unselected items**


Refer to [brush.inBrush](#brush.inBrush) and [brush.outOfBrush](#brush.outOfBrush).


  





---


Here is the configuration in detail.



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("BrushModel").tag(sync=True)
    
    id = Unicode(None, allow_none=True, help="""Component ID, not specified by default. If specified, it can be used to refer the component in option or API.""").tag(sync=True)
    
    toolbox = Any(None, allow_none=True, help="""Use the buttons in toolbox.


Buttons in toolbox that is related to brush includes:


* `'rect'`: for selection-box in rectangle shape;
* `'polygon'`: for selection-box in polygon shape;
* `'lineX'`: for horizontal selection-box;
* `'lineY'`: for vertical selection-box;
* `'keep'`: for setting mode between `single` and `multiple` selection, the former of which supports clearing selection on click, and the latter selecting multiple areas;
* `'clear'`: for clearing all selections.""").tag(sync=True)
    
    brushLink = Union([Any(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Links interaction between selected items in different series.


Following is an example of enabling selected effect for `scatter` and `parallel` charts once a scatter chart is selected.


`brushLink` is an array of `seriesIndex`es, which assigns the series that can be interacted. For example, it can be:


* `[3, 4, 5]` for interacting series with seriesIndex as `3`, `4`, or `5`;
* `'all'` for interacting all series;
* `'none'`, or `null`, or `undefined` for disabling `brushLink`.


**Attention**


`brushLink` is a mapping of `dataIndex`. So **`data` of every series with `brushLink` should be guaranteed to correspond to the other**.


Example:



```
option = {
    brush: {
        brushLink: [0, 1]
    },
    series: [
        {
            type: 'bar'
            data: [232,    4434,    545,      654]     // data has 4 items
        },
        {
            type: 'parallel',
            data: [[4, 5], [3, 5], [66, 33], [99, 66]] // data also has 4 items, which correspond to the data above
        }
    ]
};

```""").tag(sync=True)
    
    seriesIndex = Union([Any(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Assigns which of the series can use brush selecting, whose value can be:


* `'all'`: all series;
* `'Array'`: series index array;
* `'number'`: certain series index.""").tag(sync=True)
    
    geoIndex = Union([Any(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Assigns which of the geo can use brush selecting.


`brush` can be set to be *global*, or *belonging to a particular coordinate*.


**Global brushes**


Selecting is enabled for everywhere in ECharts's instance in this case. This is the default situation, when brush is not set to be global.


**Coordinate brushes**


Selecting is enabled only in the assigned coordinates in this case. Selecting-box will be altered according to scaling and translating of coordinate (see `roam` and `dataZoom`).


In practice, you may often find coordinate brush to be a more frequently made choice, particularly in `geo` charts.


By assigning [brush.geoIndex](#brush.geoIndex), or [brush.xAxisIndex](#brush.xAxisIndex), or [brush.yAxisIndex](#brush.yAxisIndex), brush selecting axes can be assigned, whose value can be:


* `'all'`: for all axes;
* `number`: like `0`, for a particular coordinate with that index;
* `Array`: like `[0, 4, 2]`, for coordinates with those indexes;
* `'none'`, or `null`, or `undefined`: for not assigning.


Example:



```
option = {
    geo: {
        ...
    },
    brush: {
        geoIndex: 'all', // brush selecting is enabled only in all geo charts above
        ...
    }
};

```

Example:



```
option = {
    grid: [
        {...}, // grid 0
        {...}  // grid 1
    ],
    xAxis: [
        {gridIndex: 1, ...}, // xAxis 0 for grid 1
        {gridIndex: 0, ...}  // xAxis 1 for grid 0
    ],
    yAxis: [
        {gridIndex: 1, ...}, // yAxis 0 for grid 1
        {gridIndex: 0, ...}  // yAxis 1 for grid 0
    ],
    brush: {
        xAxisIndex: [0, 1], // brush selecting is enabled only in coordinates with xAxisIndex as `0` or `1`
        ...
    }
};

```""").tag(sync=True)
    
    xAxisIndex = Union([Any(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Assigns which of the xAxisIndex can use brush selecting.


`brush` can be set to be *global*, or *belonging to a particular coordinate*.


**Global brushes**


Selecting is enabled for everywhere in ECharts's instance in this case. This is the default situation, when brush is not set to be global.


**Coordinate brushes**


Selecting is enabled only in the assigned coordinates in this case. Selecting-box will be altered according to scaling and translating of coordinate (see `roam` and `dataZoom`).


In practice, you may often find coordinate brush to be a more frequently made choice, particularly in `geo` charts.


By assigning [brush.geoIndex](#brush.geoIndex), or [brush.xAxisIndex](#brush.xAxisIndex), or [brush.yAxisIndex](#brush.yAxisIndex), brush selecting axes can be assigned, whose value can be:


* `'all'`: for all axes;
* `number`: like `0`, for a particular coordinate with that index;
* `Array`: like `[0, 4, 2]`, for coordinates with those indexes;
* `'none'`, or `null`, or `undefined`: for not assigning.


Example:



```
option = {
    geo: {
        ...
    },
    brush: {
        geoIndex: 'all', // brush selecting is enabled only in all geo charts above
        ...
    }
};

```

Example:



```
option = {
    grid: [
        {...}, // grid 0
        {...}  // grid 1
    ],
    xAxis: [
        {gridIndex: 1, ...}, // xAxis 0 for grid 1
        {gridIndex: 0, ...}  // xAxis 1 for grid 0
    ],
    yAxis: [
        {gridIndex: 1, ...}, // yAxis 0 for grid 1
        {gridIndex: 0, ...}  // yAxis 1 for grid 0
    ],
    brush: {
        xAxisIndex: [0, 1], // brush selecting is enabled only in coordinates with xAxisIndex as `0` or `1`
        ...
    }
};

```""").tag(sync=True)
    
    yAxisIndex = Union([Any(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Assigns which of the yAxisIndex can use brush selecting.


`brush` can be set to be *global*, or *belonging to a particular coordinate*.


**Global brushes**


Selecting is enabled for everywhere in ECharts's instance in this case. This is the default situation, when brush is not set to be global.


**Coordinate brushes**


Selecting is enabled only in the assigned coordinates in this case. Selecting-box will be altered according to scaling and translating of coordinate (see `roam` and `dataZoom`).


In practice, you may often find coordinate brush to be a more frequently made choice, particularly in `geo` charts.


By assigning [brush.geoIndex](#brush.geoIndex), or [brush.xAxisIndex](#brush.xAxisIndex), or [brush.yAxisIndex](#brush.yAxisIndex), brush selecting axes can be assigned, whose value can be:


* `'all'`: for all axes;
* `number`: like `0`, for a particular coordinate with that index;
* `Array`: like `[0, 4, 2]`, for coordinates with those indexes;
* `'none'`, or `null`, or `undefined`: for not assigning.


Example:



```
option = {
    geo: {
        ...
    },
    brush: {
        geoIndex: 'all', // brush selecting is enabled only in all geo charts above
        ...
    }
};

```

Example:



```
option = {
    grid: [
        {...}, // grid 0
        {...}  // grid 1
    ],
    xAxis: [
        {gridIndex: 1, ...}, // xAxis 0 for grid 1
        {gridIndex: 0, ...}  // xAxis 1 for grid 0
    ],
    yAxis: [
        {gridIndex: 1, ...}, // yAxis 0 for grid 1
        {gridIndex: 0, ...}  // yAxis 1 for grid 0
    ],
    brush: {
        xAxisIndex: [0, 1], // brush selecting is enabled only in coordinates with xAxisIndex as `0` or `1`
        ...
    }
};

```""").tag(sync=True)
    
    brushType = Unicode(None, allow_none=True, help="""Default type of brush.


* `'rect'`: for selection-box in rectangle shape;
* `'polygon'`: for selection-box in polygon shape;
* `'lineX'`: for horizontal selection-box;
* `'lineY'`: for vertical selection-box;""").tag(sync=True)
    
    brushMode = Unicode(None, allow_none=True, help="""Default brush mode, whose value can be:


* `'single'`: for single selection;
* `'multiple'`: for multiple selection.""").tag(sync=True)
    
    transformable = Bool(None, allow_none=True, help="""Determines whether a selected box can be changed in shape or translated.""").tag(sync=True)
    
    brushStyle = Dict(default_value=None, allow_none=True, help="""Default brush style, whose value is:



```
{
    borderWidth: 1,
    color: 'rgba(120,140,180,0.3)',
    borderColor: 'rgba(120,140,180,0.8)'
},

```""").tag(sync=True)
    
    throttleType = Unicode(None, allow_none=True, help="""By default, `brushSelected` is always triggered when selection-box is selected or moved, to tell the outside about the event.


But efficiency problems may occur when events are triggered too frequently, or the animation result may be affected. So brush components provides [brush.throttleType](#brush.throttleType) and [brush.throttleDelay](#brush.throttleDelay) to solve this problem.


Valid `throttleType` values can be:


* `'debounce'`: for triggering events only when the action has been stopped (no action after some duration). Time threshold can be assigned with [brush.throttleDelay](#brush.throttleDelay);
* `'fixRate'`: for triggering event with a certain frequency. The frequency can be assigned with [brush.throttleDelay](#brush.throttleDelay).


In this [example](https://echarts.apache.org/examples/en/view.html?c=scatter-map-brush&edit=1&reset=1), `debounce` is used to make sure the bar chart is updated only when the user has stopped action. In other cases, the animation result may not be so good.""").tag(sync=True)
    
    throttleDelay = Float(None, allow_none=True, help="""`0` for disabling throttle.


By default, `brushSelected` is always triggered when selection-box is selected or moved, to tell the outside about the event.


But efficiency problems may occur when events are triggered too frequently, or the animation result may be affected. So brush components provides [brush.throttleType](#brush.throttleType) and [brush.throttleDelay](#brush.throttleDelay) to solve this problem.


Valid `throttleType` values can be:


* `'debounce'`: for triggering events only when the action has been stopped (no action after some duration). Time threshold can be assigned with [brush.throttleDelay](#brush.throttleDelay);
* `'fixRate'`: for triggering event with a certain frequency. The frequency can be assigned with [brush.throttleDelay](#brush.throttleDelay).


In this [example](https://echarts.apache.org/examples/en/view.html?c=scatter-map-brush&edit=1&reset=1), `debounce` is used to make sure the bar chart is updated only when the user has stopped action. In other cases, the animation result may not be so good.""").tag(sync=True)
    
    removeOnClick = Bool(None, allow_none=True, help="""Defined whether *clearing all select-boxes on click* is enabled when [brush.brushMode](#brush.brushMode) is `'single'`.""").tag(sync=True)
    
    inBrush = Dict(default_value=None, allow_none=True, help="""Defines visual effects of items in selection.


Available visual effects include:


* `symbol`: Type of symbol.
* `symbolSize`: Symbol size.
* `color`: Symbol color.
* `colorAlpha`: Symbol alpha channel.
* `opacity`: Opacity of symbol and others (like labels).
* `colorLightness`: Lightness in [HSL](https://en.wikipedia.org/wiki/HSL_and_HSV).
* `colorSaturation`: Saturation in [HSL](https://en.wikipedia.org/wiki/HSL_and_HSV).
* `colorHue`: Hue in [HSL](https://en.wikipedia.org/wiki/HSL_and_HSV).


In most cases, inBrush can be left unassigned, in which case default visual configuration remains.""").tag(sync=True)
    
    outOfBrush = Dict(default_value=None, allow_none=True, help="""Defines visual effects of items out of selection.


Available visual effects include:


* `symbol`: Type of symbol.
* `symbolSize`: Symbol size.
* `color`: Symbol color.
* `colorAlpha`: Symbol alpha channel.
* `opacity`: Opacity of symbol and others (like labels).
* `colorLightness`: Lightness in [HSL](https://en.wikipedia.org/wiki/HSL_and_HSV).
* `colorSaturation`: Saturation in [HSL](https://en.wikipedia.org/wiki/HSL_and_HSV).
* `colorHue`: Hue in [HSL](https://en.wikipedia.org/wiki/HSL_and_HSV).


**Note:** If `outOfBrush` is not assigned, `color` will be set to be `'#ddd'` by default. If the color is not desired, you can use:



```
brush: {
    ...,
    outOfBrush: {
        colorAlpha: 0.1
    }
}

```""").tag(sync=True)
    
    z = Float(None, allow_none=True, help="""z-index of brush cover box. It can be adjusted when incorrect overlap occurs.""").tag(sync=True)
    
        



from traitlets import (
    Unicode, Dict, Float, Bool, Union, Any,
)
from ..basewidget import BaseWidget


class Legend(BaseWidget):
    """ 
:warning: **Autogenerated class**

Legend component.


Legend component shows symbol, color and name of different series. You can click legends to toggle displaying series in the chart.


In ECharts 3, a single echarts instance may contain multiple legend components, which makes it easier for the layout of multiple legend components.


If there have to be too many legend items, [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1) are options to paginate them. Check [legend.type](#legend.type) please.



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("LegendModel").tag(sync=True)
    
    type = Unicode(None, allow_none=True, help="""Type of legend. Optional values:


* `'plain'`: Simple legend. (default)
* `'scroll'`: Scrollable legend. It helps when too many legend items needed to be shown.


See [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1).


When `'scroll'` used, these options below can be used for detailed configuration:


* [legend.scrollDataIndex](#legend.scrollDataIndex)
* [legend.pageButtonItemGap](#legend.pageButtonItemGap)
* [legend.pageButtonGap](#legend.pageButtonGap)
* [legend.pageButtonPosition](#legend.pageButtonPosition)
* [legend.pageFormatter](#legend.pageFormatter)
* [legend.pageIcons](#legend.pageIcons)
* [legend.pageIconColor](#legend.pageIconColor)
* [legend.pageIconInactiveColor](#legend.pageIconInactiveColor)
* [legend.pageIconSize](#legend.pageIconSize)
* [legend.pageTextStyle](#legend.pageTextStyle)
* [legend.animation](#legend.animation)
* [legend.animationDurationUpdate](#legend.animationDurationUpdate)""").tag(sync=True)
    
    id = Unicode(None, allow_none=True, help="""Component ID, not specified by default. If specified, it can be used to refer the component in option or API.""").tag(sync=True)
    
    show = Bool(None, allow_none=True, ).tag(sync=True)
    
    zlevel = Float(None, allow_none=True, help="""`zlevel` value of all graphical elements in .


`zlevel` is used to make layers with Canvas. Graphical elements with different `zlevel` values will be placed in different Canvases, which is a common optimization technique. We can put those frequently changed elements (like those with animations) to a separate `zlevel`. Notice that too many Canvases will increase memory cost, and should be used carefully on mobile phones to avoid crash.


Canvases with bigger `zlevel` will be placed on Canvases with smaller `zlevel`.""").tag(sync=True)
    
    z = Float(None, allow_none=True, help="""`z` value of all graphical elements in , which controls order of drawing graphical components. Components with smaller `z` values may be overwritten by those with larger `z` values.


`z` has a lower priority to `zlevel`, and will not create new Canvas.""").tag(sync=True)
    
    left = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between legend component and the left side of the container.


`left` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`; and it can also be `'left'`, `'center'`, or `'right'`.


If the `left` value is set to be `'left'`, `'center'`, or `'right'`, then the component will be aligned automatically based on position.""").tag(sync=True)
    
    top = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between legend component and the top side of the container.


`top` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`; and it can also be `'top'`, `'middle'`, or `'bottom'`.


If the `top` value is set to be `'top'`, `'middle'`, or `'bottom'`, then the component will be aligned automatically based on position.""").tag(sync=True)
    
    right = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between legend component and the right side of the container.


`right` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`.


Adaptive by default.""").tag(sync=True)
    
    bottom = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between legend component and the bottom side of the container.


`bottom` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`.


Adaptive by default.""").tag(sync=True)
    
    width = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Width of legend component. Adaptive by default.""").tag(sync=True)
    
    height = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Height of legend component. Adaptive by default.""").tag(sync=True)
    
    orient = Unicode(None, allow_none=True, help="""The layout orientation of legend.


Options:


* 'horizontal'
* 'vertical'""").tag(sync=True)
    
    align = Unicode(None, allow_none=True, help="""Legend marker and text aligning. By default, it automatically calculates from component location and orientation. When [left](#legend.left) value of this component is 'right', and the vertical layout ([orient](#legend.orient) is 'vertical'), it would be aligned to 'right'.


Option:


* 'auto'
* 'left'
* 'right'""").tag(sync=True)
    
    padding = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""legend space around content. The unit is px. Default values for each position are 5. And they can be set to different values with left, right, top, and bottom.


Examples:



```
// Set padding to be 5
padding: 5
// Set the top and bottom paddings to be 5, and left and right paddings to be 10
padding: [5, 10]
// Set each of the four paddings separately
padding: [
    5,  // up
    10, // right
    5,  // down
    10, // left
]

```""").tag(sync=True)
    
    itemGap = Float(None, allow_none=True, help="""The distance between each legend, horizontal distance in horizontal layout, and vertical distance in vertical layout.""").tag(sync=True)
    
    itemWidth = Float(None, allow_none=True, help="""Image width of legend symbol.""").tag(sync=True)
    
    itemHeight = Float(None, allow_none=True, help="""Image height of legend symbol.""").tag(sync=True)
    
    itemStyle = Dict(default_value=None, allow_none=True, help="""Legend item style. If its children have values as `'inherit'`, the values are inherited from corresponding series options.""").tag(sync=True)
    
    lineStyle = Dict(default_value=None, allow_none=True, help="""Legend line style. If its children have values as `'inherit'`, the values are inherited from corresponding series options.""").tag(sync=True)
    
    symbolRotate = Union([Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Rotation of the symbol, which can be `number | 'inherit'`. If it's `'inherit'`, `symbolRotate` of the series will be used.""").tag(sync=True)
    
    formatter = Union([Unicode(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Formatter is used to format label of legend, which supports string template and callback function.


Example:



```
// using string template, the template variable is legend name {name}
formatter: 'Legend {name}'
// using callback function
formatter: function (name) {
    return 'Legend ' + name;
}

```""").tag(sync=True)
    
    selectedMode = Union([Unicode(default_value=None, allow_none=True),Bool(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Selected mode of legend, which controls whether series can be toggled displaying by clicking legends. It is enabled by default, and you may set it to be `false` to disable it.


Besides, it can be set to `'single'` or `'multiple'`, for single selection and multiple selection.""").tag(sync=True)
    
    inactiveColor = Any(None, allow_none=True, help="""Legend color when not selected.""").tag(sync=True)
    
    inactiveBorderColor = Any(None, allow_none=True, help="""Legend border color when not selected.""").tag(sync=True)
    
    inactiveBorderWidth = Union([Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Legend border width when not selected.


If it is `'auto'`, the border width is set to be 2 if there is border width in the series, 0 elsewise.


If it is `'inherit'`, it always takes the border width of the series.""").tag(sync=True)
    
    selected = Dict(default_value=None, allow_none=True, help="""State table of selected legend.


example:



```
selected: {
    // selected'series 1'
    'series 1': true,
    // unselected'series 2'
    'series 2': false
}

```""").tag(sync=True)
    
    textStyle = Dict(default_value=None, allow_none=True, help="""Legend text style.""").tag(sync=True)
    
    tooltip = Dict(default_value=None, allow_none=True, help="""Tooltip configuration for legend tooltip, which is similar to [tooltip](#tooltip).""").tag(sync=True)
    
    icon = Unicode(None, allow_none=True, help="""Icon of the legend items.


Icon types provided by ECharts includes


`'circle'`, `'rect'`, `'roundRect'`, `'triangle'`, `'diamond'`, `'pin'`, `'arrow'`, `'none'`


It can be set to an image with `'image://url'` , in which URL is the link to an image, or `dataURI` of an image.


An image URL example:



```
'image://http://example.website/a/b.png'

```
A `dataURI` example:



```
'image://data:image/gif;base64,R0lGODlhEAAQAMQAAORHHOVSKudfOulrSOp3WOyDZu6QdvCchPGolfO0o/XBs/fNwfjZ0frl3/zy7////wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAkAABAALAAAAAAQABAAAAVVICSOZGlCQAosJ6mu7fiyZeKqNKToQGDsM8hBADgUXoGAiqhSvp5QAnQKGIgUhwFUYLCVDFCrKUE1lBavAViFIDlTImbKC5Gm2hB0SlBCBMQiB0UjIQA7'

```
Icons can be set to arbitrary vector path via `'path://'` in ECharts. As compared with a raster image, vector paths prevent jagging and blurring when scaled, and have better control over changing colors. The size of the vector icon will be adapted automatically. Refer to [SVG PathData](http://www.w3.org/TR/SVG/paths.html#PathData) for more information about the format of the path. You may export vector paths from tools like Adobe 


For example:



```
'path://M30.9,53.2C16.8,53.2,5.3,41.7,5.3,27.6S16.8,2,30.9,2C45,2,56.4,13.5,56.4,27.6S45,53.2,30.9,53.2z M30.9,3.5C17.6,3.5,6.8,14.4,6.8,27.6c0,13.3,10.8,24.1,24.101,24.1C44.2,51.7,55,40.9,55,27.6C54.9,14.4,44.1,3.5,30.9,3.5z M36.9,35.8c0,0.601-0.4,1-0.9,1h-1.3c-0.5,0-0.9-0.399-0.9-1V19.5c0-0.6,0.4-1,0.9-1H36c0.5,0,0.9,0.4,0.9,1V35.8z M27.8,35.8 c0,0.601-0.4,1-0.9,1h-1.3c-0.5,0-0.9-0.399-0.9-1V19.5c0-0.6,0.4-1,0.9-1H27c0.5,0,0.9,0.4,0.9,1L27.8,35.8L27.8,35.8z'

```""").tag(sync=True)
    
    data = Any(None, allow_none=True, help="""Data array of legend. An array item is usually a `name` representing string. (If it is a [pie chart](#series-pie), it could also be the `name` of a single data in the pie chart) of a series. Legend component would automatically calculate the color and icon according to series. Special string `''` (null string) or `'\n'` (new line string) can be used for a new line.


If `data` is not specified, it will be auto collected from series. For most of series, it will be collected from [series.name](#series.name) or the dimension name specified by `seriesName` of [series.encode](#series.encode). For some types of series like [pie](#series-pie) and [funnel](#series-funnel), it will be collected from the name field of `series.data`.


If you need to set the style for a single item, you may also set the configuration of it. In this case, `name` attribute is used to represent name of `series`.


Example:



```
data: [{
    name: 'series 1',
    // compulsorily set icon as a circle
    icon: 'circle',
    // set up the text in red
    textStyle: {
        color: 'red'
    }
}]

```""").tag(sync=True)
    
    backgroundColor = Any(None, allow_none=True, help="""Background color of legend, which is transparent by default.



> Color can be represented in RGB, for example `'rgb(128, 128, 128)'`. RGBA can be used when you need alpha channel, for example `'rgba(128, 128, 128, 0.5)'`. You may also use hexadecimal format, for example `'#ccc'`.""").tag(sync=True)
    
    borderColor = Any(None, allow_none=True, help="""Border color of legend. Support the same color format as backgroundColor.""").tag(sync=True)
    
    borderWidth = Float(None, allow_none=True, help="""Border width of legend.""").tag(sync=True)
    
    borderRadius = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""The radius of rounded corner. Its unit is px. And it supports use array to respectively specify the 4 corner radiuses.


For example:



```
borderRadius: 5, // consistently set the size of 4 rounded corners
borderRadius: [5, 5, 0, 0] // (clockwise upper left, upper right, bottom right and bottom left)

```""").tag(sync=True)
    
    shadowBlur = Float(None, allow_none=True, help="""Size of shadow blur. This attribute should be used along with `shadowColor`,`shadowOffsetX`, `shadowOffsetY` to set shadow to component.


For example:



```
{
    shadowColor: 'rgba(0, 0, 0, 0.5)',
    shadowBlur: 10
}

```

**Attention**: This property works only if `show: true` is configured and `backgroundColor` is defined other than `transparent`.""").tag(sync=True)
    
    shadowColor = Any(None, allow_none=True, help="""Shadow color. Support same format as `color`.


**Attention**: This property works only if `show: true` configured.""").tag(sync=True)
    
    shadowOffsetX = Float(None, allow_none=True, help="""Offset distance on the horizontal direction of shadow.


**Attention**: This property works only if `show: true` configured.""").tag(sync=True)
    
    shadowOffsetY = Float(None, allow_none=True, help="""Offset distance on the vertical direction of shadow.


**Attention**: This property works only if `show: true` configured.""").tag(sync=True)
    
    scrollDataIndex = Float(None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


`dataIndex` of the left top most displayed item.


Although the scrolling of legend items can be controlled by calling `setOption` and specifying this property, we suggest that do not control legend in this way unless necessary (`setOption` might be time-consuming), but just use action [legendScroll](api.html#action.legend.legendScroll) to do that.


See [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1).""").tag(sync=True)
    
    pageButtonItemGap = Float(None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


The gap between page buttons and page info text.


See [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1).""").tag(sync=True)
    
    pageButtonGap = Float(None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


The gap between page buttons and legend items.


See [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1).""").tag(sync=True)
    
    pageButtonPosition = Unicode(None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


The position of page buttons and page info. Optional values:


* `'start'`: on the left or top.
* `'end'`: on the right or bottom.


See [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1).""").tag(sync=True)
    
    pageFormatter = Union([Unicode(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


Page info formatter. It is `'{current}/{total}'` by default, where `{current}` is current page number (start from 1), and `{total}` is the total page number.


If `pageFormatter` is a function, it should return a string. The parameters is:



```
{
    current: number
    total: number
}

```

See [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1).""").tag(sync=True)
    
    pageIcons = Dict(default_value=None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


The icons of page buttons.""").tag(sync=True)
    
    pageIconColor = Unicode(None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


The color of page buttons.


See [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1).""").tag(sync=True)
    
    pageIconInactiveColor = Unicode(None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


The color of page buttons when they are inactive.


See [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1).""").tag(sync=True)
    
    pageIconSize = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


The size of page buttons. It can be a number, or an array, like `[10, 3]`, represents `[width, height]`.


See [vertically scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=pie-legend&edit=1&reset=1) or [horizontally scrollable legend](https://echarts.apache.org/examples/en/editor.html?c=radar2&edit=1&reset=1).""").tag(sync=True)
    
    pageTextStyle = Dict(default_value=None, allow_none=True, help="""It works when [legend.type](#legend.type) is `'scroll'`.


The text style of page info.""").tag(sync=True)
    
    animation = Bool(None, allow_none=True, help="""Whether to use animation when page scrolls.""").tag(sync=True)
    
    animationDurationUpdate = Float(None, allow_none=True, help="""Duration of the page scroll animation.""").tag(sync=True)
    
    emphasis = Dict(default_value=None, allow_none=True, ).tag(sync=True)
    
    selector = Union([Bool(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""> Since `v4.4.0`


The selector button in the legend component. Currently, there are two types of button:


* `all`: Select All
* `inverse`: Inverse Selection


The selector button doesn't display by default, you need to enable it manually as follows.



```
selector: [
    {
        type: 'all',
        // can be any title you like
        title: 'All'
    },
    {
        type: 'inverse',
        // can be any title you like
        title: 'Inv'
    }
]

// or
selector: true

// or
selector: ['all', 'inverse']

```""").tag(sync=True)
    
    selectorLabel = Dict(default_value=None, allow_none=True, help="""> Since `v4.4.0`


The text label style of the selector button, which is displayed by default.""").tag(sync=True)
    
    selectorPosition = Unicode(None, allow_none=True, help="""> Since `v4.4.0`


The position of the selector button, which can be placed at the end or start of the legend component, the corresponding values are `'end'` and `'start'`. By default, when the legend is laid out horizontally, the selector is placed at the end of it, and when the legend is laid out vertically, the selector is placed at the start of it.""").tag(sync=True)
    
    selectorItemGap = Float(None, allow_none=True, help="""> Since `v4.4.0`


The gap between the selector button.""").tag(sync=True)
    
    selectorButtonGap = Float(None, allow_none=True, help="""> Since `v4.4.0`


The gap between selector button and legend component.""").tag(sync=True)
    
        



from ipywidgets import (
    widget_serialization
)
from traitlets import (
    Unicode, Instance, List, Union,
)
from ..basewidget import BaseWidget

from .seriesitems.line import Line
from .seriesitems.bar import Bar
from .seriesitems.pie import Pie
from .seriesitems.scatter import Scatter
from .seriesitems.effectscatter import EffectScatter
from .seriesitems.seriesradar import SeriesRadar
from .seriesitems.tree import Tree
from .seriesitems.treemap import Treemap
from .seriesitems.sunburst import Sunburst
from .seriesitems.boxplot import Boxplot
from .seriesitems.candlestick import Candlestick
from .seriesitems.heatmap import Heatmap
from .seriesitems.map import Map
from .seriesitems.seriesparallel import SeriesParallel
from .seriesitems.lines import Lines
from .seriesitems.graph import Graph
from .seriesitems.sankey import Sankey
from .seriesitems.funnel import Funnel
from .seriesitems.gauge import Gauge
from .seriesitems.pictorialbar import PictorialBar
from .seriesitems.themeriver import ThemeRiver
from .seriesitems.custom import Custom
from .seriesitems.scatter3d import Scatter3D
from .seriesitems.bar3d import Bar3D
from .seriesitems.line3d import Line3D
from .seriesitems.lines3d import Lines3D
from .seriesitems.map3d import Map3D
from .seriesitems.surface import Surface
from .seriesitems.polygons3d import Polygons3D
from .seriesitems.scattergl import ScatterGL
from .seriesitems.graphgl import GraphGL
from .seriesitems.flowgl import FlowGL

class Series(BaseWidget):
    """ 
:warning: **Autogenerated class**


    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("SeriesModel").tag(sync=True)
    
    series = List(trait=Union([Instance(Line, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Bar, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Pie, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Scatter, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(EffectScatter, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(SeriesRadar, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Tree, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Treemap, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Sunburst, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Boxplot, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Candlestick, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Heatmap, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Map, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(SeriesParallel, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Lines, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Graph, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Sankey, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Funnel, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Gauge, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(PictorialBar, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(ThemeRiver, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Custom, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Scatter3D, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Bar3D, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Line3D, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Lines3D, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Map3D, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Surface, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(Polygons3D, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(ScatterGL, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(GraphGL, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),Instance(FlowGL, kw=None, args=None, allow_none=True).tag(sync=True, **widget_serialization),]), allow_none=True, default_value=None).tag(sync=True, **widget_serialization)
    
        



from traitlets import (
    Unicode, Dict, Float, Bool, Any,
)
from ..basewidget import BaseWidget


class Bar3D(BaseWidget):
    """ 
:warning: **Autogenerated class**

3D bar. It can be used to display data in [grid3D](#grid3D),  [geo3D](#geo3D), [globe](#globe) using attributes such as size 、color and so on.


The figure below shows the population density data of the world through a 3D bar chart on [geo3D] (~geo3D).


![](documents/asset/gl/img/geo-bar3D.jpg)



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("Bar3DModel").tag(sync=True)
    
    type = Unicode("bar3D", allow_none=True, ).tag(sync=True)
    
    name = Unicode(None, allow_none=True, help="""Series name used for displaying in [tooltip](https://echarts.apache.org/zh/option.html#tooltip) and filtering with [legend](https://echarts.apache.org/zh/option.html#legend), or updating data and configuration with `setOption`.""").tag(sync=True)
    
    coordinateSystem = Unicode(None, allow_none=True, help="""The coordinate used in the series, whose options are:


* `'cartesian3D'`


 Use a 3D rectangular coordinate (also known as Cartesian coordinate), with [xAxisIndex](#series-.xAxisIndex) and [yAxisIndex](#series-.yAxisIndex) to assign the corresponding axis component.


* `'geo3D'`
 Use 3D geographic coordinate, with [geoIndex](#series-.geoIndex) to assign the corresponding 3D geographic coordinate components.


* `'globe'`


 Use 3D globe coordinate, with [globeIndex](#series-.globeIndex) to assign the corresponding 3D globe coordinate components.""").tag(sync=True)
    
    grid3DIndex = Float(None, allow_none=True, help="""Use the index of the [grid3D](#grid3D) component. The first [grid3D](#grid3D) component is used by default.""").tag(sync=True)
    
    geo3DIndex = Float(None, allow_none=True, help="""The index of the [geo3D](#geo3D) component used by the axis.The first [grid3D](#grid3D) component is used by default.""").tag(sync=True)
    
    globeIndex = Float(None, allow_none=True, help="""The index of the [globe](#globe) component used by the axis.The first [globe](#globe) component is used by default.""").tag(sync=True)
    
    bevelSize = Float(None, allow_none=True, help="""The size of the bevel.
Support for setting values from 0 to 1. The default is 0, which means there is no bevel.


Below are the differences between beveling and no beveling.



![](documents/asset/gl/img/bar3D-no-bevel.png "bevelSize: 0")
![](documents/asset/gl/img/bar3D-bevel.png "bevelSize: 0.3")""").tag(sync=True)
    
    bevelSmoothness = Float(None, allow_none=True, help="""The smoothness of the bevel, the larger the value, the smoother.""").tag(sync=True)
    
    stack = Unicode(None, allow_none=True, help="""Stacking of bar chart. On the same category axis, the series with the same `stack` name would be put on top of each other. Note that the data items that need to be stack in different series must have the same index in the array.


See also [stackStrategy](#series-bar3D.stackStrategy) on how to customize how values are stacked.


Notice: `stack` only supports stacking on `value` and `log` axis for now. `time` and `category` axis are not supported.""").tag(sync=True)
    
    stackStrategy = Unicode(None, allow_none=True, help="""> Since ECharts `v5.3.3`


How to stack values if the [stack](#series-bar3D.stack) property has been set. Options:


* `'samesign'`: only stack values if the value to be stacked has the same sign as the currently cumulated stacked value.
* `'all'`: stack all values, irrespective of the signs of the current or cumulative stacked value.
* `'positive'`: only stack positive values.
* `'negative'`: only stack negative values.""").tag(sync=True)
    
    minHeight = Float(None, allow_none=True, help="""The minimum width of the bar.""").tag(sync=True)
    
    itemStyle = Dict(default_value=None, allow_none=True, help="""The style of the bar, including color and opacity.""").tag(sync=True)
    
    label = Dict(default_value=None, allow_none=True, help="""Configure the label of the bar.""").tag(sync=True)
    
    emphasis = Dict(default_value=None, allow_none=True, help="""Configure labels and styles for bar highlighting.""").tag(sync=True)
    
    data = Any(None, allow_none=True, help="""A data array of 3D bar. Each item of the array is a piece of data. Usually this data is an array to store each attribute/dimension of the data. For example below:



```
data: [
    [[12, 14, 10], [34, 50, 15], [56, 30, 20], [10, 15, 12], [23, 10, 14]]
]

```

For each item in the array:


1. In [grid3D] (~grid3D), the first three values of each item are `x`, `y`, `z`.
2. In [geo3D](#geo3D) and [globe](#globe), the first two values of each item are longitude `lng`, latitude `lat`, and the third value indicates the size of the value, such as the number of people. This value will be mapped to the range of [minHeight](#series-bar3D.minHeight) ~ [maxHeight](#series-bar3D.maxHeight).


In addition to the three values used by default for the coordinate system, each item can be added with any number of values for mapping the [visualMap](#visualMap) component to other graphical properties such as color.


More likely, we need to assign name to each data item, in which case each item should be an object:



```
[{
    // name of date item
    name: 'data1',
    // value of date item is 8
    value: [12, 14, 10]
}, {
    name: 'data2',
    value: 20
}]

```

Each data item can be further customized:



```
[{
    name: 'data1',
    value: [12, 14, 10]
}, {
    // name of data item
    name: 'data2',
    value : [34, 50, 15],
    // user-defined special itemStyle that only useful for this data item
    itemStyle:{}
}]

```""").tag(sync=True)
    
    shading = Unicode(None, allow_none=True, help="""The coloring effect of 3D graphics in 3D Bar. The following three coloring methods are supported in echarts-gl:


* `'color'`
Only display colors, not affected by other factors such as lighting.
* `'lambert'`
Through the classic [lambert] coloring, can express the light and dark that the light shows.
* `'realistic'`
Realistic rendering, combined with [light.ambientCubemap](#globe.light.ambientCubemap) and [postEffect](#globe.postEffect), can improve the quality and texture of the display. [Physical Based Rendering (PBR)] (<https://www.marmoset.co/posts/physically-based-rendering-and-you-can-too/>) is used in ECharts GL to represent realistic materials.""").tag(sync=True)
    
    realisticMaterial = Dict(default_value=None, allow_none=True, help="""The configuration item of the realistic material is valid when [shading](#series-bar3D.shading) is `'realistic'`.""").tag(sync=True)
    
    lambertMaterial = Dict(default_value=None, allow_none=True, help="""The configuration item of the lambert material is valid when [shading](#series-bar3D.shading) is `'lambert'`.""").tag(sync=True)
    
    colorMaterial = Dict(default_value=None, allow_none=True, help="""The color material related configuration item is valid when [shading](#series-bar3D.shading) is `'color'`.""").tag(sync=True)
    
    zlevel = Float(None, allow_none=True, help="""The layer in which the component is located.


`zlevel` is used to make layers with Canvas. Graphical elements with different `zlevel` values will be placed in different Canvases, which is a common optimization technique. We can put those frequently changed elements (like those with animations) to a separate `zlevel`. Notice that too many Canvases will increase memory cost, and should be used carefully on mobile phones to avoid the crash.


Canvases with bigger `zlevel` will be placed on Canvases with smaller `zlevel`.


**Note:** The layers of the components in echarts-gl need to be separated from the layers of the components in echarts. The same `zlevel` cannot be used for both WebGL and Canvas drawing at the same time.""").tag(sync=True)
    
    silent = Bool(None, allow_none=True, help="""Whether the graph doesn`t respond and triggers a mouse event. The default is false, which is to respond to and trigger mouse events.""").tag(sync=True)
    
    animation = Bool(None, allow_none=True, help="""Whether to enable animation.""").tag(sync=True)
    
    animationDurationUpdate = Float(None, allow_none=True, help="""The duration time for update the transition animation.""").tag(sync=True)
    
    animationEasingUpdate = Unicode(None, allow_none=True, help="""The easing effect for update transition animation.""").tag(sync=True)
    
        



from traitlets import (
    Unicode, Dict, Float, Bool, Union, Any,
)
from ..basewidget import BaseWidget


class Gauge(BaseWidget):
    """ 
:warning: **Autogenerated class**

**Gauge chart**


**Example:** 




    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("GaugeModel").tag(sync=True)
    
    type = Unicode("gauge", allow_none=True, ).tag(sync=True)
    
    id = Unicode(None, allow_none=True, help="""Component ID, not specified by default. If specified, it can be used to refer the component in option or API.""").tag(sync=True)
    
    name = Unicode(None, allow_none=True, help="""Series name used for displaying in [tooltip](#tooltip) and filtering with [legend](#legend), or updating data and configuration with `setOption`.""").tag(sync=True)
    
    colorBy = Unicode(None, allow_none=True, help="""> Since `v5.2.0`


The policy to take color from [option.color](#color). Valid values:


* `'series'`: assigns the colors in the palette by series, so that all data in the same series are in the same color;
* `'data'`: assigns colors in the palette according to data items, with each data item using a different color.""").tag(sync=True)
    
    zlevel = Float(None, allow_none=True, help="""`zlevel` value of all graphical elements in .


`zlevel` is used to make layers with Canvas. Graphical elements with different `zlevel` values will be placed in different Canvases, which is a common optimization technique. We can put those frequently changed elements (like those with animations) to a separate `zlevel`. Notice that too many Canvases will increase memory cost, and should be used carefully on mobile phones to avoid crash.


Canvases with bigger `zlevel` will be placed on Canvases with smaller `zlevel`.""").tag(sync=True)
    
    z = Float(None, allow_none=True, help="""`z` value of all graphical elements in , which controls order of drawing graphical components. Components with smaller `z` values may be overwritten by those with larger `z` values.


`z` has a lower priority to `zlevel`, and will not create new Canvas.""").tag(sync=True)
    
    center = Any(None, allow_none=True, help="""Center position of , the first of which is the horizontal position, and the second is the vertical position.


Percentage is supported. When set in percentage, the item is relative to the container width, and the second item to the height.


**Example:** 



```
// Set to absolute pixel values
center: [400, 300]
// Set to relative percent
center: ['50%', '50%']

```""").tag(sync=True)
    
    radius = Union([Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""The radius of gauge chart. It can be a percentage value of the smaller of container half width and half height, also can be an absolute value.""").tag(sync=True)
    
    legendHoverLink = Bool(None, allow_none=True, help="""Whether to enable highlighting chart when [legend](#legend) is being hovered.""").tag(sync=True)
    
    startAngle = Float(None, allow_none=True, help="""The start angle of gauge chart. The direct right side of [circle center](#series-gauge.center) is `0` degree, the right above it is `90` degree, the direct left side of it is `180` degree.""").tag(sync=True)
    
    endAngle = Float(None, allow_none=True, help="""The end angle of gauge chart.""").tag(sync=True)
    
    clockwise = Bool(None, allow_none=True, help="""Whether the scale in gauge chart increases clockwise.""").tag(sync=True)
    
    data = Any(None, allow_none=True, help="""Data array of series, which can be a single data value, like:



```
[12, 34, 56, 10, 23]

```

Or, if need extra dimensions for components like [visualMap](#visualMap) to map to graphic attributes like color, it can also be in the form of array. For example:



```
[[12, 14], [34, 50], [56, 30], [10, 15], [23, 10]]

```

In this case, we can assign the second value in each array item to [visualMap](#visualMap) component.


More likely, we need to assign name to each data item, in which case each item should be an object:



```
[{
    // name of date item
    name: 'data1',
    // value of date item is 8
    value: 10
}, {
    name: 'data2',
    value: 20
}]

```

Each data item can be further customized:



```
[{
    name: 'data1',
    value: 10
}, {
    // name of data item
    name: 'data2',
    value : 56,
    // user-defined label format that only useful for this data item
    label: {},
    // user-defined special itemStyle that only useful for this data item
    itemStyle:{}
}]

```""").tag(sync=True)
    
    min = Float(None, allow_none=True, help="""The minimum data value which map to [minAngle](#series-gauge.minAngle).""").tag(sync=True)
    
    max = Float(None, allow_none=True, help="""The maximum data value which map to [maxAngle](#series-gauge.maxAngle).""").tag(sync=True)
    
    splitNumber = Float(None, allow_none=True, help="""The number of split segments of gauge chart scale.""").tag(sync=True)
    
    axisLine = Dict(default_value=None, allow_none=True, help="""The related configuration about the axis line of gauge chart.""").tag(sync=True)
    
    progress = Dict(default_value=None, allow_none=True, help="""> Since `v5.0`


Used to show current progress.""").tag(sync=True)
    
    splitLine = Dict(default_value=None, allow_none=True, help="""The style of split line.""").tag(sync=True)
    
    axisTick = Dict(default_value=None, allow_none=True, help="""The tick line style.""").tag(sync=True)
    
    axisLabel = Dict(default_value=None, allow_none=True, help="""Axis tick label.""").tag(sync=True)
    
    pointer = Dict(default_value=None, allow_none=True, help="""Gauge chart pointer.""").tag(sync=True)
    
    anchor = Dict(default_value=None, allow_none=True, help="""> Since `v5.0`


The fixed point of a pointer in a dial.""").tag(sync=True)
    
    itemStyle = Dict(default_value=None, allow_none=True, help="""The style of gauge chart.""").tag(sync=True)
    
    emphasis = Dict(default_value=None, allow_none=True, help="""Configurations of emphasis state.""").tag(sync=True)
    
    title = Dict(default_value=None, allow_none=True, help="""The title of gauge chart.""").tag(sync=True)
    
    detail = Dict(default_value=None, allow_none=True, help="""The detail about gauge chart which is used to show data.""").tag(sync=True)
    
    markPoint = Dict(default_value=None, allow_none=True, help="""Mark point in a chart.""").tag(sync=True)
    
    markLine = Dict(default_value=None, allow_none=True, help="""Use a line in the chart to illustrate.""").tag(sync=True)
    
    markArea = Dict(default_value=None, allow_none=True, help="""Used to mark an area in chart. For example, mark a time interval.""").tag(sync=True)
    
    silent = Bool(None, allow_none=True, help="""Whether to ignore mouse events. Default value is false, for triggering and responding to mouse events.""").tag(sync=True)
    
    animation = Bool(None, allow_none=True, help="""Whether to enable animation.""").tag(sync=True)
    
    animationThreshold = Float(None, allow_none=True, help="""Whether to set graphic number threshold to animation. Animation will be disabled when graphic number is larger than threshold.""").tag(sync=True)
    
    animationDuration = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Duration of the first animation, which supports callback function for different data to have different animation effect:



```
animationDuration: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```""").tag(sync=True)
    
    animationEasing = Unicode(None, allow_none=True, help="""Easing method used for the first animation. Varied easing effects can be found at [easing effect example](https://echarts.apache.org/examples/en/editor.html?c=line-easing).""").tag(sync=True)
    
    animationDelay = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Delay before updating the first animation, which supports callback function for different data to have different animation effect.


For example:



```
animationDelay: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```

See [this example](https://echarts.apache.org/examples/en/editor.html?c=bar-animation-delay) for more information.""").tag(sync=True)
    
    animationDurationUpdate = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Time for animation to complete, which supports callback function for different data to have different animation effect:



```
animationDurationUpdate: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```""").tag(sync=True)
    
    animationEasingUpdate = Unicode(None, allow_none=True, help="""Easing method used for animation.""").tag(sync=True)
    
    animationDelayUpdate = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Delay before updating animation, which supports callback function for different data to have different animation effects.


For example:



```
animationDelayUpdate: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```

See [this example](https://echarts.apache.org/examples/en/editor.html?c=bar-animation-delay) for more information.""").tag(sync=True)
    
    tooltip = Dict(default_value=None, allow_none=True, help="""tooltip settings in this series.""").tag(sync=True)
    
        



from traitlets import (
    Unicode, Dict, Float, Bool, Union, Any,
)
from ..basewidget import BaseWidget


class YAxis3D(BaseWidget):
    """ 
:warning: **Autogenerated class**

The Y-axis in a 3D cartesian coordinate system.
You can use [grid3DIndex](#yAxis3D.grid3DIndex) to index [3D cartesian coordinate system](#grid3D).


The [axisLine](#yAxis3D.axisLine), [axisTick](#yAxis3D.axisTick), [axisLabel](#yAxis3D.axisLabel), [splitLine](#yAxis3D.splitLine), [splitArea](#yAxis3D.splitArea), [axisPointer](#yAxis3D.axisPointer) set separately on [yAxis3D](#yAxis3D) will cover the corresponding configuration items under [grid3D](#grid3D).



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("YAxis3DModel").tag(sync=True)
    
    show = Bool(None, allow_none=True, help="""Whether to display the y-axis.""").tag(sync=True)
    
    name = Unicode(None, allow_none=True, help="""The name of the axis.""").tag(sync=True)
    
    grid3DIndex = Float(None, allow_none=True, help="""The index of the [grid3D](#grid3D) component used by the axis. The default is to use the first [grid3D](#grid3D) component.""").tag(sync=True)
    
    nameTextStyle = Dict(default_value=None, allow_none=True, help="""Text style of axis name.""").tag(sync=True)
    
    nameGap = Float(None, allow_none=True, help="""The gap between the axis name and axis line. Note the distance is in 3D space, not the screen pixel value.""").tag(sync=True)
    
    type = Unicode(None, allow_none=True, help="""The type of the axis.


Optional:


* `'value'`
 The value axis. Suitable for continuous data.
* `'category'`
 The category axis. Suitable for the discrete category data. For this type, the category data must be set through [data](#yAxis3D.data).
* `'time'`
 The timeline. Suitable for the continuous timing data. The time axis has a time format compared to the value axis, and the scale calculation is also different. For example, the scale of the month, week, day, and hour ranges can be determined according to the range of the span.
* `'log'`
 Logarithmic axis. Suitable for the logarithmic data.""").tag(sync=True)
    
    min = Union([Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""The minimum value of axis.


It can be set to a special value `'dataMin'` so that the minimum value on this axis is set to be the minimum label.


It will be automatically computed to make sure the axis tick is equally distributed when not set.


In the category axis, it can also be set as the ordinal number. For example, if a category axis has `data: ['categoryA', 'categoryB', 'categoryC']`, and the ordinal `2` represents `'categoryC'`. Moreover, it can be set as a negative number, like `-3`.""").tag(sync=True)
    
    max = Union([Float(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""The maximum value of the axis.


It can be set to a special value `'dataMax'` so that the minimum value on this axis is set to be the maximum label.


It will be automatically computed to make sure the axis tick is equally distributed when not set.


In the category axis, it can also be set as the ordinal number. For example, if a category axis has `data: ['categoryA', 'categoryB', 'categoryC']`, and the ordinal `2` represents `'categoryC'`. Moreover, it can be set as a negative number, like `-3`.""").tag(sync=True)
    
    scale = Bool(None, allow_none=True, help="""It is available only in numerical axis, i.e., [type](#yAxis3D.type): 'value'.


It specifies whether not to contain zero position of axis compulsively. When it is set to be `true`, the axis may not contain zero position, which is useful in the scatter chart for both value axes.


This configuration item is unavailable when the [min](#yAxis3D.min) and [max](#yAxis3D.max) are set.""").tag(sync=True)
    
    splitNumber = Float(None, allow_none=True, help="""The number of segments that the axis is split into. Note that this number serves only as a recommendation, and the true segments may be adjusted based on readability.


This is unavailable for the category axis.""").tag(sync=True)
    
    minInterval = Float(None, allow_none=True, help="""Minimum gap between split lines.


For example, it can be set to be `1` to make sure the axis label is shown as an integer.



```
{
    minInterval: 1
}

```

It is available only for axis of [type](#yAxis3D.type) 'value' or 'time'.""").tag(sync=True)
    
    interval = Float(None, allow_none=True, help="""Compulsively set segmentation interval for axis.


As [splitNumber](#yAxis3D.splitNumber) is a recommendation value, the calculated tick may not be the same as expected. In this case, interval should be used along with [min](#yAxis3D.min) and [max](#yAxis3D.max) to compulsively set tickings. But in most cases, we do not suggest using this, out automatic calculation is enough for most situations.


This is unavailable for category axis. Timestamp should be passed for [type](#yAxis3D.type): 'time' axis. Logged value should be passed for [type](#yAxis3D.type): 'log' axis.""").tag(sync=True)
    
    logBase = Float(None, allow_none=True, help="""Base of logarithm, which is valid only for numeric axes with [type](#yAxis3D.type): 'log'.""").tag(sync=True)
    
    data = Any(None, allow_none=True, help="""Category data, available in [type](#yAxis3D.type): 'category' axis.


If [type](#yAxis3D.type) is specified as `'category'`, but `axis.data` is not specified, `axis.data` will be auto collected from [series.data](#series.data). It brings convenience, but we should notice that `axis.data` provides then value range of the `'category'` axis. If it is auto collected from [series.data](#series.data), Only the values appearing in [series.data](#series.data) can be collected. For example, if [series.data](#series.data) is empty, nothing will be collected.


Example:



```
// Name list of all categories
data: ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']
// Each item could also be a specific configuration item.
// In this case, `value` is used as the category name.
data: [{
    value: 'Monday',
    // Highlight Monday
    textStyle: {
        fontSize: 20,
        color: 'red'
    }
}, 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']

```""").tag(sync=True)
    
    axisLine = Dict(default_value=None, allow_none=True, help="""Settings related to axis line.""").tag(sync=True)
    
    axisLabel = Dict(default_value=None, allow_none=True, help="""Settings related to axis label.""").tag(sync=True)
    
    axisTick = Dict(default_value=None, allow_none=True, help="""Settings related to axis tick.""").tag(sync=True)
    
    splitLine = Dict(default_value=None, allow_none=True, help="""Settings related to axis line.""").tag(sync=True)
    
    splitArea = Dict(default_value=None, allow_none=True, help="""Split area of axis in [grid](#grid) area.""").tag(sync=True)
    
    axisPointer = Dict(default_value=None, allow_none=True, help="""Configurations for axis pointer.""").tag(sync=True)
    
        



from traitlets import (
    Unicode, Dict, Float, Bool, Union, Any,
)
from ..basewidget import BaseWidget


class Sankey(BaseWidget):
    """ 
:warning: **Autogenerated class**

 **Sankey diagram** 
Sankey diagram is a specific type of streamgraph (can also be seen as a directed acyclic graph) in which the width of each branch is shown proportionally to the flow quantity. These graphs are typically used to visualize energy or material or cost transfers between processes. They can also visualize the energy accounts, material flow accounts on a regional or national level, and also the breakdown of cost of item or services.


**Example:** 



  

**Visual Encoding:** 


The Sankey diagram encodes each `node` of the raw data into a small rectangle. Different nodes are presented in different colors as far as possible. The `label` next to the small rectangle encodes the name of the node.


In addition, the edge between two small rectangles in the diagram encodes the `link` of the raw data. The width of edge is shown proportionally to the `value` of `link`.



    """

    def __init__(self, **kwargs):

        super().__init__(**kwargs)  
    
    _model_name = Unicode("SankeyModel").tag(sync=True)
    
    type = Unicode("sankey", allow_none=True, ).tag(sync=True)
    
    id = Unicode(None, allow_none=True, help="""Component ID, not specified by default. If specified, it can be used to refer the component in option or API.""").tag(sync=True)
    
    name = Unicode(None, allow_none=True, help="""Series name used for displaying in [tooltip](#tooltip) and filtering with [legend](#legend), or updating data and configuration with `setOption`.""").tag(sync=True)
    
    zlevel = Float(None, allow_none=True, help="""`zlevel` value of all graphical elements in .


`zlevel` is used to make layers with Canvas. Graphical elements with different `zlevel` values will be placed in different Canvases, which is a common optimization technique. We can put those frequently changed elements (like those with animations) to a separate `zlevel`. Notice that too many Canvases will increase memory cost, and should be used carefully on mobile phones to avoid crash.


Canvases with bigger `zlevel` will be placed on Canvases with smaller `zlevel`.""").tag(sync=True)
    
    z = Float(None, allow_none=True, help="""`z` value of all graphical elements in , which controls order of drawing graphical components. Components with smaller `z` values may be overwritten by those with larger `z` values.


`z` has a lower priority to `zlevel`, and will not create new Canvas.""").tag(sync=True)
    
    left = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between sankey component and the left side of the container.


`left` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`; and it can also be `'left'`, `'center'`, or `'right'`.


If the `left` value is set to be `'left'`, `'center'`, or `'right'`, then the component will be aligned automatically based on position.""").tag(sync=True)
    
    top = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between sankey component and the top side of the container.


`top` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`; and it can also be `'top'`, `'middle'`, or `'bottom'`.


If the `top` value is set to be `'top'`, `'middle'`, or `'bottom'`, then the component will be aligned automatically based on position.""").tag(sync=True)
    
    right = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between sankey component and the right side of the container.


`right` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`.""").tag(sync=True)
    
    bottom = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Distance between sankey component and the bottom side of the container.


`bottom` can be a pixel value like `20`; it can also be a percentage value relative to container width like `'20%'`.""").tag(sync=True)
    
    width = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Width of sankey component.""").tag(sync=True)
    
    height = Union([Unicode(default_value=None, allow_none=True),Float(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Height of sankey component.""").tag(sync=True)
    
    nodeWidth = Float(None, allow_none=True, help="""The node width of rectangle in Sankey diagram.""").tag(sync=True)
    
    nodeGap = Float(None, allow_none=True, help="""The gap between any two rectangles in each column of the Sankey diagram.""").tag(sync=True)
    
    nodeAlign = Unicode(None, allow_none=True, help="""Controls the horizontal alignment of nodes in the diagram.


May be:


* `left`: initial nodes (those with no incoming links) are aligned to the left of the diagram.
* `right`: terminal nodes (those with no outgoing links) are aligned to the right of the diagram.
* `justify`: initial and terminal nodes are aligned on the left and right edges.


Note when [orient](#series-sankey.orient) is `vertical`, `nodeAlign` controls vertical alignment.""").tag(sync=True)
    
    layoutIterations = Float(None, allow_none=True, help="""The iterations of layout, which is used to iteratively optimize the position of the nodes and edges in the Sankey diagram to reduce the overlapping between nodes and edges. The default value is `32`. If you want the order of the nodes in the chart to be the same with the order in the original [data](#series-sankey.data), you can set the value to be `0`.""").tag(sync=True)
    
    orient = Unicode(None, allow_none=True, help="""The layout direction of the nodes in the Sankey diagram, which can be horizontal from left to right or vertical from top to bottom. The corresponding parameter values ​​are `horizontal` or `vertical`.""").tag(sync=True)
    
    draggable = Bool(None, allow_none=True, help="""The drag-and-drop interaction of the node, which is enabled by default. After opening, the user can drag any node in the Sankey diagram to any position. To turn this interaction off, simply set the value to `false`.""").tag(sync=True)
    
    edgeLabel = Dict(default_value=None, allow_none=True, help="""> Since `v5.4.1`


The label style of each edge/link.""").tag(sync=True)
    
    levels = Any(None, allow_none=True, help="""The setting of each layer of Sankey diagram. Can be set layer by layer, as follows:



```
levels: [{
    depth: 0,
    itemStyle: {
        color: '#fbb4ae'
    },
    lineStyle: {
        color: 'source',
        opacity: 0.6
    }
}, {
    depth: 1,
    itemStyle: {
        color: '#b3cde3'
    },
    lineStyle: {
        color: 'source',
        opacity: 0.6
    }
}]

```

You can also only set a certain layer:



```
levels: [{
    depth: 3,
    itemStyle: {
        color: '#fbb4ae'
    },
    lineStyle: {
        color: 'source',
        opacity: 0.6
    }
}]

```""").tag(sync=True)
    
    label = Dict(default_value=None, allow_none=True, help="""`label` describes the text label style in each rectangular node.""").tag(sync=True)
    
    labelLayout = Union([Any(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""> Since `v5.0.0`


Unified layout configuration of labels.


It provide a chance to adjust the labels' `(x, y)` position, alignment based on the original layout each series provides.


This option can be a callback with following parameters.



```
// corresponding index of data
dataIndex: number
// corresponding type of data. Only available in graph, in which it can be 'node' or 'edge'
dataType?: string
// corresponding index of series
seriesIndex: number
// Displayed text of label.
text: string
// Bounding rectangle of label.
labelRect: {x: number, y: number, width: number, height: number}
// Horizontal alignment of label.
align: 'left' | 'center' | 'right'
// Vertical alignment of label.
verticalAlign: 'top' | 'middle' | 'bottom'
// Bounding rectangle of the element corresponding to.
rect: {x: number, y: number, width: number, height: number}
// Default points array of labelLine. Currently only provided in pie and funnel series.
// It's null in other series.
labelLinePoints?: number[][]

```

**Example:**


Align the labels on the right. Left 10px margin to the edge.



```
labelLayout(params) {
    return {
        x: params.rect.x + 10,
        y: params.rect.y + params.rect.height / 2,
        verticalAlign: 'middle',
        align: 'left'
    }
}

```

Set the text size based on the size of element bounding rectangle.



```

labelLayout(params) {
    return {
        fontSize: Math.max(params.rect.width / 10, 5)
    };
}

```""").tag(sync=True)
    
    itemStyle = Dict(default_value=None, allow_none=True, help="""The style of node rectangle in Sankey diagram.""").tag(sync=True)
    
    lineStyle = Dict(default_value=None, allow_none=True, help="""The edge style of Sankey diagram""").tag(sync=True)
    
    emphasis = Dict(default_value=None, allow_none=True, help="""Configurations of emphasis state.""").tag(sync=True)
    
    blur = Dict(default_value=None, allow_none=True, help="""> Since `v5.0.0`


Configurations of blur state. Available when [emphasis.focus](#series-sankey.emphasis.focus) is set.""").tag(sync=True)
    
    select = Dict(default_value=None, allow_none=True, help="""> Since `v5.0.0`


Configurations of selected state. Available when [selectedMode](#series-sankey.selectedMode) is set.""").tag(sync=True)
    
    selectedMode = Union([Bool(default_value=None, allow_none=True),Unicode(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""> Since `v5.0.0`


Selected mode. It is enabled by default, and you may set it to be `false` to disable it.


Besides, it can be set to `'single'`, `'multiple'` or `'series'`, for single selection, multiple selections and whole series selection.



> `'series'` is supported since v5.3.0""").tag(sync=True)
    
    data = Any(None, allow_none=True, help="""The nodes list of the sankey diagram.



```
data: [{
    name: 'node1',
    // This attribute decides the layer of the current node.
    depth: 0
}, {
    name: 'node2',
    depth: 1
}]

```

**Notice:** The name of the node cannot be repeated.""").tag(sync=True)
    
    nodes = Any(None, allow_none=True, help="""Equals to [data](#series-sankey.data)""").tag(sync=True)
    
    links = Any(None, allow_none=True, help="""The links between nodes. **Notes: The Sankey diagram theoretically only supports Directed Acyclic Graph(DAG), so please make sure that there is no cycle in the links.** For instance:



```
links: [{
    source: 'n1',
    target: 'n2'
}, {
    source: 'n2',
    target: 'n3'
}]

```""").tag(sync=True)
    
    edges = Any(None, allow_none=True, help="""Equals to [links](#series-sankey.links)""").tag(sync=True)
    
    silent = Bool(None, allow_none=True, help="""Whether to ignore mouse events. Default value is false, for triggering and responding to mouse events.""").tag(sync=True)
    
    animation = Bool(None, allow_none=True, help="""Whether to enable animation.""").tag(sync=True)
    
    animationThreshold = Float(None, allow_none=True, help="""Whether to set graphic number threshold to animation. Animation will be disabled when graphic number is larger than threshold.""").tag(sync=True)
    
    animationDuration = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Duration of the first animation, which supports callback function for different data to have different animation effect:



```
animationDuration: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```""").tag(sync=True)
    
    animationEasing = Unicode(None, allow_none=True, help="""Easing method used for the first animation. Varied easing effects can be found at [easing effect example](https://echarts.apache.org/examples/en/editor.html?c=line-easing).""").tag(sync=True)
    
    animationDelay = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Delay before updating the first animation, which supports callback function for different data to have different animation effect.


For example:



```
animationDelay: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```

See [this example](https://echarts.apache.org/examples/en/editor.html?c=bar-animation-delay) for more information.""").tag(sync=True)
    
    animationDurationUpdate = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Time for animation to complete, which supports callback function for different data to have different animation effect:



```
animationDurationUpdate: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```""").tag(sync=True)
    
    animationEasingUpdate = Unicode(None, allow_none=True, help="""Easing method used for animation.""").tag(sync=True)
    
    animationDelayUpdate = Union([Float(default_value=None, allow_none=True),Any(default_value=None, allow_none=True),], default_value=None, allow_none=True, help="""Delay before updating animation, which supports callback function for different data to have different animation effects.


For example:



```
animationDelayUpdate: function (idx) {
    // delay for later data is larger
    return idx * 100;
}

```

See [this example](https://echarts.apache.org/examples/en/editor.html?c=bar-animation-delay) for more information.""").tag(sync=True)
    
    tooltip = Dict(default_value=None, allow_none=True, help="""tooltip settings in this series.""").tag(sync=True)
    
        
